"""
Python module generated from Java source file java.util.Calendar

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import OptionalDataException
from java.io import Serializable
from java.security import AccessControlContext
from java.security import AccessController
from java.security import PermissionCollection
from java.security import PrivilegedActionException
from java.security import PrivilegedExceptionAction
from java.security import ProtectionDomain
from java.text import DateFormat
from java.text import DateFormatSymbols
from java.time import Instant
from java.util import *
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent import ConcurrentMap
from sun.util import BuddhistCalendar
from sun.util.calendar import ZoneInfo
from sun.util.locale.provider import CalendarDataUtility
from sun.util.locale.provider import LocaleProviderAdapter
from sun.util.locale.provider import TimeZoneNameUtility
from sun.util.spi import CalendarProvider
from typing import Any, Callable, Iterable, Tuple


class Calendar(Serializable, Cloneable, Comparable):
    """
    The `Calendar` class is an abstract class that provides methods
    for converting between a specific instant in time and a set of .fields calendar fields such as `YEAR`, `MONTH`,
    `DAY_OF_MONTH`, `HOUR`, and so on, and for
    manipulating the calendar fields, such as getting the date of the next
    week. An instant in time can be represented by a millisecond value that is
    an offset from the <a id="Epoch">*Epoch*</a>, January 1, 1970
    00:00:00.000 GMT (Gregorian).
    
    The class also provides additional fields and methods for
    implementing a concrete calendar system outside the package. Those
    fields and methods are defined as `protected`.
    
    
    Like other locale-sensitive classes, `Calendar` provides a
    class method, `getInstance`, for getting a generally useful
    object of this type. `Calendar`'s `getInstance` method
    returns a `Calendar` object whose
    calendar fields have been initialized with the current date and time:
    <blockquote>
    ```
        Calendar rightNow = Calendar.getInstance();
    ```
    </blockquote>
    
    A `Calendar` object can produce all the calendar field values
    needed to implement the date-time formatting for a particular language and
    calendar style (for example, Japanese-Gregorian, Japanese-Traditional).
    `Calendar` defines the range of values returned by
    certain calendar fields, as well as their meaning.  For example,
    the first month of the calendar system has value `MONTH ==
    JANUARY` for all calendars.  Other values are defined by the
    concrete subclass, such as `ERA`.  See individual field
    documentation and subclass documentation for details.
    
    <h2>Getting and Setting Calendar Field Values</h2>
    
    The calendar field values can be set by calling the `set`
    methods. Any field values set in a `Calendar` will not be
    interpreted until it needs to calculate its time value (milliseconds from
    the Epoch) or values of the calendar fields. Calling the
    `get`, `getTimeInMillis`, `getTime`,
    `add` and `roll` involves such calculation.
    
    <h3>Leniency</h3>
    
    `Calendar` has two modes for interpreting the calendar
    fields, *lenient* and *non-lenient*.  When a
    `Calendar` is in lenient mode, it accepts a wider range of
    calendar field values than it produces.  When a `Calendar`
    recomputes calendar field values for return by `get()`, all of
    the calendar fields are normalized. For example, a lenient
    `GregorianCalendar` interprets `MONTH == JANUARY`,
    `DAY_OF_MONTH == 32` as February 1.
    
    When a `Calendar` is in non-lenient mode, it throws an
    exception if there is any inconsistency in its calendar fields. For
    example, a `GregorianCalendar` always produces
    `DAY_OF_MONTH` values between 1 and the length of the month. A
    non-lenient `GregorianCalendar` throws an exception upon
    calculating its time or calendar field values if any out-of-range field
    value has been set.
    
    <h3><a id="first_week">First Week</a></h3>
    
    `Calendar` defines a locale-specific seven day week using two
    parameters: the first day of the week and the minimal days in first week
    (from 1 to 7).  These numbers are taken from the locale resource data or the
    locale itself when a `Calendar` is constructed. If the designated
    locale contains "fw" and/or "rg" <a href="./Locale.html#def_locale_extension">
    Unicode extensions</a>, the first day of the week will be obtained according to
    those extensions. If both "fw" and "rg" are specified, the value from the "fw"
    extension supersedes the implicit one from the "rg" extension.
    They may also be specified explicitly through the methods for setting their
    values.
    
    When setting or getting the `WEEK_OF_MONTH` or
    `WEEK_OF_YEAR` fields, `Calendar` must determine the
    first week of the month or year as a reference point.  The first week of a
    month or year is defined as the earliest seven day period beginning on
    `getFirstDayOfWeek()` and containing at least
    `getMinimalDaysInFirstWeek()` days of that month or year.  Weeks
    numbered ..., -1, 0 precede the first week; weeks numbered 2, 3,... follow
    it.  Note that the normalized numbering returned by `get()` may be
    different.  For example, a specific `Calendar` subclass may
    designate the week before week 1 of a year as week `*n*` of
    the previous year.
    
    <h3>Calendar Fields Resolution</h3>
    
    When computing a date and time from the calendar fields, there
    may be insufficient information for the computation (such as only
    year and month with no day of month), or there may be inconsistent
    information (such as Tuesday, July 15, 1996 (Gregorian) -- July 15,
    1996 is actually a Monday). `Calendar` will resolve
    calendar field values to determine the date and time in the
    following way.
    
    <a id="resolution">If there is any conflict in calendar field values,
    `Calendar` gives priorities to calendar fields that have been set
    more recently.</a> The following are the default combinations of the
    calendar fields. The most recent combination, as determined by the
    most recently set single field, will be used.
    
    <a id="date_resolution">For the date fields</a>:
    <blockquote>
    ```
    YEAR + MONTH + DAY_OF_MONTH
    YEAR + MONTH + WEEK_OF_MONTH + DAY_OF_WEEK
    YEAR + MONTH + DAY_OF_WEEK_IN_MONTH + DAY_OF_WEEK
    YEAR + DAY_OF_YEAR
    YEAR + DAY_OF_WEEK + WEEK_OF_YEAR
    ```</blockquote>
    
    <a id="time_resolution">For the time of day fields</a>:
    <blockquote>
    ```
    HOUR_OF_DAY
    AM_PM + HOUR
    ```</blockquote>
    
    If there are any calendar fields whose values haven't been set in the selected
    field combination, `Calendar` uses their default values. The default
    value of each field may vary by concrete calendar systems. For example, in
    `GregorianCalendar`, the default of a field is the same as that
    of the start of the Epoch: i.e., `YEAR = 1970`, `MONTH =
    JANUARY`, `DAY_OF_MONTH = 1`, etc.
    
    
    <strong>Note:</strong> There are certain possible ambiguities in
    interpretation of certain singular times, which are resolved in the
    following ways:
    <ol>
        -  23:59 is the last minute of the day and 00:00 is the first
             minute of the next day. Thus, 23:59 on Dec 31, 1999 &lt; 00:00 on
             Jan 1, 2000 &lt; 00:01 on Jan 1, 2000.
    
        -  Although historically not precise, midnight also belongs to "am",
             and noon belongs to "pm", so on the same day,
             12:00 am (midnight) &lt; 12:01 am, and 12:00 pm (noon) &lt; 12:01 pm
    </ol>
    
    
    The date or time format strings are not part of the definition of a
    calendar, as those must be modifiable or overridable by the user at
    runtime. Use DateFormat
    to format dates.
    
    <h3>Field Manipulation</h3>
    
    The calendar fields can be changed using three methods:
    `set()`, `add()`, and `roll()`.
    
    <strong>`set(f, value)`</strong> changes calendar field
    `f` to `value`.  In addition, it sets an
    internal member variable to indicate that calendar field `f` has
    been changed. Although calendar field `f` is changed immediately,
    the calendar's time value in milliseconds is not recomputed until the next call to
    `get()`, `getTime()`, `getTimeInMillis()`,
    `add()`, or `roll()` is made. Thus, multiple calls to
    `set()` do not trigger multiple, unnecessary
    computations. As a result of changing a calendar field using
    `set()`, other calendar fields may also change, depending on the
    calendar field, the calendar field value, and the calendar system. In addition,
    `get(f)` will not necessarily return `value` set by
    the call to the `set` method
    after the calendar fields have been recomputed. The specifics are determined by
    the concrete calendar class.
    
    *Example*: Consider a `GregorianCalendar`
    originally set to August 31, 1999. Calling `set(Calendar.MONTH,
    Calendar.SEPTEMBER)` sets the date to September 31,
    1999. This is a temporary internal representation that resolves to
    October 1, 1999 if `getTime()` is then called. However, a
    call to `set(Calendar.DAY_OF_MONTH, 30)` before the call to
    `getTime()` sets the date to September 30, 1999, since
    no recomputation occurs after `set()` itself.
    
    <strong>`add(f, delta)`</strong> adds `delta`
    to field `f`.  This is equivalent to calling `set(f,
    get(f) + delta)` with two adjustments:
    
    <blockquote>
      <strong>Add rule 1</strong>. The value of field `f`
      after the call minus the value of field `f` before the
      call is `delta`, modulo any overflow that has occurred in
      field `f`. Overflow occurs when a field value exceeds its
      range and, as a result, the next larger field is incremented or
      decremented and the field value is adjusted back into its range.
    
      <strong>Add rule 2</strong>. If a smaller field is expected to be
      invariant, but it is impossible for it to be equal to its
      prior value because of changes in its minimum or maximum after field
      `f` is changed or other constraints, such as time zone
      offset changes, then its value is adjusted to be as close
      as possible to its expected value. A smaller field represents a
      smaller unit of time. `HOUR` is a smaller field than
      `DAY_OF_MONTH`. No adjustment is made to smaller fields
      that are not expected to be invariant. The calendar system
      determines what fields are expected to be invariant.
    </blockquote>
    
    In addition, unlike `set()`, `add()` forces
    an immediate recomputation of the calendar's milliseconds and all
    fields.
    
    *Example*: Consider a `GregorianCalendar`
    originally set to August 31, 1999. Calling `add(Calendar.MONTH,
    13)` sets the calendar to September 30, 2000. <strong>Add rule
    1</strong> sets the `MONTH` field to September, since
    adding 13 months to August gives September of the next year. Since
    `DAY_OF_MONTH` cannot be 31 in September in a
    `GregorianCalendar`, <strong>add rule 2</strong> sets the
    `DAY_OF_MONTH` to 30, the closest possible value. Although
    it is a smaller field, `DAY_OF_WEEK` is not adjusted by
    rule 2, since it is expected to change when the month changes in a
    `GregorianCalendar`.
    
    <strong>`roll(f, delta)`</strong> adds
    `delta` to field `f` without changing larger
    fields. This is equivalent to calling `add(f, delta)` with
    the following adjustment:
    
    <blockquote>
      <strong>Roll rule</strong>. Larger fields are unchanged after the
      call. A larger field represents a larger unit of
      time. `DAY_OF_MONTH` is a larger field than
      `HOUR`.
    </blockquote>
    
    *Example*: See java.util.GregorianCalendar.roll(int, int).
    
    <strong>Usage model</strong>. To motivate the behavior of
    `add()` and `roll()`, consider a user interface
    component with increment and decrement buttons for the month, day, and
    year, and an underlying `GregorianCalendar`. If the
    interface reads January 31, 1999 and the user presses the month
    increment button, what should it read? If the underlying
    implementation uses `set()`, it might read March 3, 1999. A
    better result would be February 28, 1999. Furthermore, if the user
    presses the month increment button again, it should read March 31,
    1999, not March 28, 1999. By saving the original date and using either
    `add()` or `roll()`, depending on whether larger
    fields should be affected, the user interface can behave as most users
    will intuitively expect.

    Author(s)
    - Mark Davis, David Goldsmith, Chen-Lieh Huang, Alan Liu

    See
    - java.text.DateFormat

    Since
    - 1.1
    """

    ERA = 0
    """
    Field number for `get` and `set` indicating the
    era, e.g., AD or BC in the Julian calendar. This is a calendar-specific
    value; see subclass documentation.

    See
    - GregorianCalendar.BC
    """
    YEAR = 1
    """
    Field number for `get` and `set` indicating the
    year. This is a calendar-specific value; see subclass documentation.
    """
    MONTH = 2
    """
    Field number for `get` and `set` indicating the
    month. This is a calendar-specific value. The first month of
    the year in the Gregorian and Julian calendars is
    `JANUARY` which is 0; the last depends on the number
    of months in a year.

    See
    - .UNDECIMBER
    """
    WEEK_OF_YEAR = 3
    """
    Field number for `get` and `set` indicating the
    week number within the current year.  The first week of the year, as
    defined by `getFirstDayOfWeek()` and
    `getMinimalDaysInFirstWeek()`, has value 1.  Subclasses define
    the value of `WEEK_OF_YEAR` for days before the first week of
    the year.

    See
    - .getMinimalDaysInFirstWeek
    """
    WEEK_OF_MONTH = 4
    """
    Field number for `get` and `set` indicating the
    week number within the current month.  The first week of the month, as
    defined by `getFirstDayOfWeek()` and
    `getMinimalDaysInFirstWeek()`, has value 1.  Subclasses define
    the value of `WEEK_OF_MONTH` for days before the first week of
    the month.

    See
    - .getMinimalDaysInFirstWeek
    """
    DATE = 5
    """
    Field number for `get` and `set` indicating the
    day of the month. This is a synonym for `DAY_OF_MONTH`.
    The first day of the month has value 1.

    See
    - .DAY_OF_MONTH
    """
    DAY_OF_MONTH = 5
    """
    Field number for `get` and `set` indicating the
    day of the month. This is a synonym for `DATE`.
    The first day of the month has value 1.

    See
    - .DATE
    """
    DAY_OF_YEAR = 6
    """
    Field number for `get` and `set` indicating the day
    number within the current year.  The first day of the year has value 1.
    """
    DAY_OF_WEEK = 7
    """
    Field number for `get` and `set` indicating the day
    of the week.  This field takes values `SUNDAY`,
    `MONDAY`, `TUESDAY`, `WEDNESDAY`,
    `THURSDAY`, `FRIDAY`, and `SATURDAY`.

    See
    - .SATURDAY
    """
    DAY_OF_WEEK_IN_MONTH = 8
    """
    Field number for `get` and `set` indicating the
    ordinal number of the day of the week within the current month. Together
    with the `DAY_OF_WEEK` field, this uniquely specifies a day
    within a month.  Unlike `WEEK_OF_MONTH` and
    `WEEK_OF_YEAR`, this field's value does *not* depend on
    `getFirstDayOfWeek()` or
    `getMinimalDaysInFirstWeek()`.  `DAY_OF_MONTH 1`
    through `7` always correspond to `DAY_OF_WEEK_IN_MONTH
    1`; `8` through `14` correspond to
    `DAY_OF_WEEK_IN_MONTH 2`, and so on.
    `DAY_OF_WEEK_IN_MONTH 0` indicates the week before
    `DAY_OF_WEEK_IN_MONTH 1`.  Negative values count back from the
    end of the month, so the last Sunday of a month is specified as
    `DAY_OF_WEEK = SUNDAY, DAY_OF_WEEK_IN_MONTH = -1`.  Because
    negative values count backward they will usually be aligned differently
    within the month than positive values.  For example, if a month has 31
    days, `DAY_OF_WEEK_IN_MONTH -1` will overlap
    `DAY_OF_WEEK_IN_MONTH 5` and the end of `4`.

    See
    - .WEEK_OF_MONTH
    """
    AM_PM = 9
    """
    Field number for `get` and `set` indicating
    whether the `HOUR` is before or after noon.
    E.g., at 10:04:15.250 PM the `AM_PM` is `PM`.

    See
    - .HOUR
    """
    HOUR = 10
    """
    Field number for `get` and `set` indicating the
    hour of the morning or afternoon. `HOUR` is used for the
    12-hour clock (0 - 11). Noon and midnight are represented by 0, not by 12.
    E.g., at 10:04:15.250 PM the `HOUR` is 10.

    See
    - .HOUR_OF_DAY
    """
    HOUR_OF_DAY = 11
    """
    Field number for `get` and `set` indicating the
    hour of the day. `HOUR_OF_DAY` is used for the 24-hour clock.
    E.g., at 10:04:15.250 PM the `HOUR_OF_DAY` is 22.

    See
    - .HOUR
    """
    MINUTE = 12
    """
    Field number for `get` and `set` indicating the
    minute within the hour.
    E.g., at 10:04:15.250 PM the `MINUTE` is 4.
    """
    SECOND = 13
    """
    Field number for `get` and `set` indicating the
    second within the minute.
    E.g., at 10:04:15.250 PM the `SECOND` is 15.
    """
    MILLISECOND = 14
    """
    Field number for `get` and `set` indicating the
    millisecond within the second.
    E.g., at 10:04:15.250 PM the `MILLISECOND` is 250.
    """
    ZONE_OFFSET = 15
    """
    Field number for `get` and `set`
    indicating the raw offset from GMT in milliseconds.
    
    This field reflects the correct GMT offset value of the time
    zone of this `Calendar` if the
    `TimeZone` implementation subclass supports
    historical GMT offset changes.
    """
    DST_OFFSET = 16
    """
    Field number for `get` and `set` indicating the
    daylight saving offset in milliseconds.
    
    This field reflects the correct daylight saving offset value of
    the time zone of this `Calendar` if the
    `TimeZone` implementation subclass supports
    historical Daylight Saving Time schedule changes.
    """
    FIELD_COUNT = 17
    """
    The number of distinct fields recognized by `get` and `set`.
    Field numbers range from `0..FIELD_COUNT-1`.
    """
    SUNDAY = 1
    """
    Value of the .DAY_OF_WEEK field indicating
    Sunday.
    """
    MONDAY = 2
    """
    Value of the .DAY_OF_WEEK field indicating
    Monday.
    """
    TUESDAY = 3
    """
    Value of the .DAY_OF_WEEK field indicating
    Tuesday.
    """
    WEDNESDAY = 4
    """
    Value of the .DAY_OF_WEEK field indicating
    Wednesday.
    """
    THURSDAY = 5
    """
    Value of the .DAY_OF_WEEK field indicating
    Thursday.
    """
    FRIDAY = 6
    """
    Value of the .DAY_OF_WEEK field indicating
    Friday.
    """
    SATURDAY = 7
    """
    Value of the .DAY_OF_WEEK field indicating
    Saturday.
    """
    JANUARY = 0
    """
    Value of the .MONTH field indicating the
    first month of the year in the Gregorian and Julian calendars.
    """
    FEBRUARY = 1
    """
    Value of the .MONTH field indicating the
    second month of the year in the Gregorian and Julian calendars.
    """
    MARCH = 2
    """
    Value of the .MONTH field indicating the
    third month of the year in the Gregorian and Julian calendars.
    """
    APRIL = 3
    """
    Value of the .MONTH field indicating the
    fourth month of the year in the Gregorian and Julian calendars.
    """
    MAY = 4
    """
    Value of the .MONTH field indicating the
    fifth month of the year in the Gregorian and Julian calendars.
    """
    JUNE = 5
    """
    Value of the .MONTH field indicating the
    sixth month of the year in the Gregorian and Julian calendars.
    """
    JULY = 6
    """
    Value of the .MONTH field indicating the
    seventh month of the year in the Gregorian and Julian calendars.
    """
    AUGUST = 7
    """
    Value of the .MONTH field indicating the
    eighth month of the year in the Gregorian and Julian calendars.
    """
    SEPTEMBER = 8
    """
    Value of the .MONTH field indicating the
    ninth month of the year in the Gregorian and Julian calendars.
    """
    OCTOBER = 9
    """
    Value of the .MONTH field indicating the
    tenth month of the year in the Gregorian and Julian calendars.
    """
    NOVEMBER = 10
    """
    Value of the .MONTH field indicating the
    eleventh month of the year in the Gregorian and Julian calendars.
    """
    DECEMBER = 11
    """
    Value of the .MONTH field indicating the
    twelfth month of the year in the Gregorian and Julian calendars.
    """
    UNDECIMBER = 12
    """
    Value of the .MONTH field indicating the
    thirteenth month of the year. Although `GregorianCalendar`
    does not use this value, lunar calendars do.
    """
    AM = 0
    """
    Value of the .AM_PM field indicating the
    period of the day from midnight to just before noon.
    """
    PM = 1
    """
    Value of the .AM_PM field indicating the
    period of the day from noon to just before midnight.
    """
    ALL_STYLES = 0
    """
    A style specifier for .getDisplayNames(int, int, Locale)
    getDisplayNames indicating names in all styles, such as
    "January" and "Jan".

    See
    - .LONG

    Since
    - 1.6
    """
    SHORT = 1
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames equivalent to .SHORT_FORMAT.

    See
    - .LONG

    Since
    - 1.6
    """
    LONG = 2
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames equivalent to .LONG_FORMAT.

    See
    - .SHORT

    Since
    - 1.6
    """
    NARROW_FORMAT = 4
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames indicating a narrow name used for format. Narrow names
    are typically single character strings, such as "M" for Monday.

    See
    - .LONG_FORMAT

    Since
    - 1.8
    """
    NARROW_STANDALONE = NARROW_FORMAT | STANDALONE_MASK
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames indicating a narrow name independently. Narrow names
    are typically single character strings, such as "M" for Monday.

    See
    - .LONG_STANDALONE

    Since
    - 1.8
    """
    SHORT_FORMAT = 1
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames indicating a short name used for format.

    See
    - .LONG_STANDALONE

    Since
    - 1.8
    """
    LONG_FORMAT = 2
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames indicating a long name used for format.

    See
    - .SHORT_STANDALONE

    Since
    - 1.8
    """
    SHORT_STANDALONE = SHORT | STANDALONE_MASK
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames indicating a short name used independently,
    such as a month abbreviation as calendar headers.

    See
    - .LONG_STANDALONE

    Since
    - 1.8
    """
    LONG_STANDALONE = LONG | STANDALONE_MASK
    """
    A style specifier for .getDisplayName(int, int, Locale)
    getDisplayName and .getDisplayNames(int, int, Locale)
    getDisplayNames indicating a long name used independently,
    such as a month name as calendar headers.

    See
    - .SHORT_STANDALONE

    Since
    - 1.8
    """


    @staticmethod
    def getInstance() -> "Calendar":
        """
        Gets a calendar using the default time zone and locale. The
        `Calendar` returned is based on the current time
        in the default time zone with the default
        Locale.Category.FORMAT FORMAT locale.
        
        If the locale contains the time zone with "tz"
        <a href="Locale.html#def_locale_extension">Unicode extension</a>,
        that time zone is used instead.

        Returns
        - a Calendar.
        """
        ...


    @staticmethod
    def getInstance(zone: "TimeZone") -> "Calendar":
        """
        Gets a calendar using the specified time zone and default locale.
        The `Calendar` returned is based on the current time
        in the given time zone with the default
        Locale.Category.FORMAT FORMAT locale.

        Arguments
        - zone: the time zone to use

        Returns
        - a Calendar.
        """
        ...


    @staticmethod
    def getInstance(aLocale: "Locale") -> "Calendar":
        """
        Gets a calendar using the default time zone and specified locale.
        The `Calendar` returned is based on the current time
        in the default time zone with the given locale.
        
        If the locale contains the time zone with "tz"
        <a href="Locale.html#def_locale_extension">Unicode extension</a>,
        that time zone is used instead.

        Arguments
        - aLocale: the locale for the week data

        Returns
        - a Calendar.
        """
        ...


    @staticmethod
    def getInstance(zone: "TimeZone", aLocale: "Locale") -> "Calendar":
        """
        Gets a calendar with the specified time zone and locale.
        The `Calendar` returned is based on the current time
        in the given time zone with the given locale.

        Arguments
        - zone: the time zone to use
        - aLocale: the locale for the week data

        Returns
        - a Calendar.
        """
        ...


    @staticmethod
    def getAvailableLocales() -> list["Locale"]:
        """
        Returns an array of all locales for which the `getInstance`
        methods of this class can return localized instances.
        The array returned must contain at least a `Locale`
        instance equal to java.util.Locale.US Locale.US.

        Returns
        - An array of locales for which localized
                `Calendar` instances are available.
        """
        ...


    def getTime(self) -> "Date":
        """
        Returns a `Date` object representing this
        `Calendar`'s time value (millisecond offset from the <a
        href="#Epoch">Epoch</a>").

        Returns
        - a `Date` representing the time value.

        See
        - .getTimeInMillis()
        """
        ...


    def setTime(self, date: "Date") -> None:
        """
        Sets this Calendar's time with the given `Date`.
        
        Note: Calling `setTime()` with
        `Date(Long.MAX_VALUE)` or `Date(Long.MIN_VALUE)`
        may yield incorrect field values from `get()`.

        Arguments
        - date: the given Date.

        Raises
        - NullPointerException: if `date` is `null`

        See
        - .setTimeInMillis(long)
        """
        ...


    def getTimeInMillis(self) -> int:
        """
        Returns this Calendar's time value in milliseconds.

        Returns
        - the current time as UTC milliseconds from the epoch.

        See
        - .setTimeInMillis(long)
        """
        ...


    def setTimeInMillis(self, millis: int) -> None:
        """
        Sets this Calendar's current time from the given long value.

        Arguments
        - millis: the new time in UTC milliseconds from the epoch.

        See
        - .getTimeInMillis()
        """
        ...


    def get(self, field: int) -> int:
        """
        Returns the value of the given calendar field. In lenient mode,
        all calendar fields are normalized. In non-lenient mode, all
        calendar fields are validated and this method throws an
        exception if any calendar fields have out-of-range values. The
        normalization and validation are handled by the
        .complete() method, which process is calendar
        system dependent.

        Arguments
        - field: the given calendar field.

        Returns
        - the value for the given calendar field.

        Raises
        - ArrayIndexOutOfBoundsException: if the specified field is out of range
                    (`field &lt; 0 || field &gt;= FIELD_COUNT`).

        See
        - .complete()
        """
        ...


    def set(self, field: int, value: int) -> None:
        """
        Sets the given calendar field to the given value. The value is not
        interpreted by this method regardless of the leniency mode.

        Arguments
        - field: the given calendar field.
        - value: the value to be set for the given calendar field.

        Raises
        - ArrayIndexOutOfBoundsException: if the specified field is out of range
                    (`field &lt; 0 || field &gt;= FIELD_COUNT`).
        in non-lenient mode.

        See
        - .get(int)
        """
        ...


    def set(self, year: int, month: int, date: int) -> None:
        """
        Sets the values for the calendar fields `YEAR`,
        `MONTH`, and `DAY_OF_MONTH`.
        Previous values of other calendar fields are retained.  If this is not desired,
        call .clear() first.

        Arguments
        - year: the value used to set the `YEAR` calendar field.
        - month: the value used to set the `MONTH` calendar field.
        Month value is 0-based. e.g., 0 for January.
        - date: the value used to set the `DAY_OF_MONTH` calendar field.

        See
        - .set(int,int,int,int,int,int)
        """
        ...


    def set(self, year: int, month: int, date: int, hourOfDay: int, minute: int) -> None:
        """
        Sets the values for the calendar fields `YEAR`,
        `MONTH`, `DAY_OF_MONTH`,
        `HOUR_OF_DAY`, and `MINUTE`.
        Previous values of other fields are retained.  If this is not desired,
        call .clear() first.

        Arguments
        - year: the value used to set the `YEAR` calendar field.
        - month: the value used to set the `MONTH` calendar field.
        Month value is 0-based. e.g., 0 for January.
        - date: the value used to set the `DAY_OF_MONTH` calendar field.
        - hourOfDay: the value used to set the `HOUR_OF_DAY` calendar field.
        - minute: the value used to set the `MINUTE` calendar field.

        See
        - .set(int,int,int,int,int,int)
        """
        ...


    def set(self, year: int, month: int, date: int, hourOfDay: int, minute: int, second: int) -> None:
        """
        Sets the values for the fields `YEAR`, `MONTH`,
        `DAY_OF_MONTH`, `HOUR_OF_DAY`, `MINUTE`, and
        `SECOND`.
        Previous values of other fields are retained.  If this is not desired,
        call .clear() first.

        Arguments
        - year: the value used to set the `YEAR` calendar field.
        - month: the value used to set the `MONTH` calendar field.
        Month value is 0-based. e.g., 0 for January.
        - date: the value used to set the `DAY_OF_MONTH` calendar field.
        - hourOfDay: the value used to set the `HOUR_OF_DAY` calendar field.
        - minute: the value used to set the `MINUTE` calendar field.
        - second: the value used to set the `SECOND` calendar field.

        See
        - .set(int,int,int,int,int)
        """
        ...


    def clear(self) -> None:
        """
        Sets all the calendar field values and the time value
        (millisecond offset from the <a href="#Epoch">Epoch</a>) of
        this `Calendar` undefined. This means that .isSet(int) isSet() will return `False` for all the
        calendar fields, and the date and time calculations will treat
        the fields as if they had never been set. A
        `Calendar` implementation class may use its specific
        default field values for date/time calculations. For example,
        `GregorianCalendar` uses 1970 if the
        `YEAR` field value is undefined.

        See
        - .clear(int)
        """
        ...


    def clear(self, field: int) -> None:
        """
        Sets the given calendar field value and the time value
        (millisecond offset from the <a href="#Epoch">Epoch</a>) of
        this `Calendar` undefined. This means that .isSet(int) isSet(field) will return `False`, and
        the date and time calculations will treat the field as if it
        had never been set. A `Calendar` implementation
        class may use the field's specific default value for date and
        time calculations.
        
        The .HOUR_OF_DAY, .HOUR and .AM_PM
        fields are handled independently and the <a
        href="#time_resolution">the resolution rule for the time of
        day</a> is applied. Clearing one of the fields doesn't reset
        the hour of day value of this `Calendar`. Use .set(int,int) set(Calendar.HOUR_OF_DAY, 0) to reset the hour
        value.

        Arguments
        - field: the calendar field to be cleared.

        See
        - .clear()
        """
        ...


    def isSet(self, field: int) -> bool:
        """
        Determines if the given calendar field has a value set,
        including cases that the value has been set by internal fields
        calculations triggered by a `get` method call.

        Arguments
        - field: the calendar field to test

        Returns
        - `True` if the given calendar field has a value set;
        `False` otherwise.
        """
        ...


    def getDisplayName(self, field: int, style: int, locale: "Locale") -> str:
        """
        Returns the string representation of the calendar
        `field` value in the given `style` and
        `locale`.  If no string representation is
        applicable, `null` is returned. This method calls
        Calendar.get(int) get(field) to get the calendar
        `field` value if the string representation is
        applicable to the given calendar `field`.
        
        For example, if this `Calendar` is a
        `GregorianCalendar` and its date is 2005-01-01, then
        the string representation of the .MONTH field would be
        "January" in the long style in an English locale or "Jan" in
        the short style. However, no string representation would be
        available for the .DAY_OF_MONTH field, and this method
        would return `null`.
        
        The default implementation supports the calendar fields for
        which a DateFormatSymbols has names in the given
        `locale`.

        Arguments
        - field: the calendar field for which the string representation
               is returned
        - style: the style applied to the string representation; one of .SHORT_FORMAT (.SHORT), .SHORT_STANDALONE,
               .LONG_FORMAT (.LONG), .LONG_STANDALONE,
               .NARROW_FORMAT, or .NARROW_STANDALONE.
        - locale: the locale for the string representation
               (any calendar types specified by `locale` are ignored)

        Returns
        - the string representation of the given
               `field` in the given `style`, or
               `null` if no string representation is
               applicable.

        Raises
        - IllegalArgumentException: if `field` or `style` is invalid,
               or if this `Calendar` is non-lenient and any
               of the calendar fields have invalid values
        - NullPointerException: if `locale` is null

        Since
        - 1.6
        """
        ...


    def getDisplayNames(self, field: int, style: int, locale: "Locale") -> dict[str, "Integer"]:
        """
        Returns a `Map` containing all names of the calendar
        `field` in the given `style` and
        `locale` and their corresponding field values. For
        example, if this `Calendar` is a GregorianCalendar, the returned map would contain "Jan" to
        .JANUARY, "Feb" to .FEBRUARY, and so on, in the
        .SHORT short style in an English locale.
        
        Narrow names may not be unique due to use of single characters,
        such as "S" for Sunday and Saturday. In that case narrow names are not
        included in the returned `Map`.
        
        The values of other calendar fields may be taken into
        account to determine a set of display names. For example, if
        this `Calendar` is a lunisolar calendar system and
        the year value given by the .YEAR field has a leap
        month, this method would return month names containing the leap
        month name, and month names are mapped to their values specific
        for the year.
        
        The default implementation supports display names contained in
        a DateFormatSymbols. For example, if `field`
        is .MONTH and `style` is .ALL_STYLES, this method returns a `Map` containing
        all strings returned by DateFormatSymbols.getShortMonths()
        and DateFormatSymbols.getMonths().

        Arguments
        - field: the calendar field for which the display names are returned
        - style: the style applied to the string representation; one of .SHORT_FORMAT (.SHORT), .SHORT_STANDALONE,
               .LONG_FORMAT (.LONG), .LONG_STANDALONE,
               .NARROW_FORMAT, or .NARROW_STANDALONE
        - locale: the locale for the display names

        Returns
        - a `Map` containing all display names in
               `style` and `locale` and their
               field values, or `null` if no display names
               are defined for `field`

        Raises
        - IllegalArgumentException: if `field` or `style` is invalid,
               or if this `Calendar` is non-lenient and any
               of the calendar fields have invalid values
        - NullPointerException: if `locale` is null

        Since
        - 1.6
        """
        ...


    @staticmethod
    def getAvailableCalendarTypes() -> set[str]:
        """
        Returns an unmodifiable `Set` containing all calendar types
        supported by `Calendar` in the runtime environment. The available
        calendar types can be used for the <a
        href="Locale.html#def_locale_extension">Unicode locale extensions</a>.
        The `Set` returned contains at least `"gregory"`. The
        calendar types don't include aliases, such as `"gregorian"` for
        `"gregory"`.

        Returns
        - an unmodifiable `Set` containing all available calendar types

        See
        - Locale.getUnicodeLocaleType(String)

        Since
        - 1.8
        """
        ...


    def getCalendarType(self) -> str:
        """
        Returns the calendar type of this `Calendar`. Calendar types are
        defined by the *Unicode Locale Data Markup Language (LDML)*
        specification.
        
        The default implementation of this method returns the class name of
        this `Calendar` instance. Any subclasses that implement
        LDML-defined calendar systems should override this method to return
        appropriate calendar types.

        Returns
        - the LDML-defined calendar type or the class name of this
                `Calendar` instance

        See
        - Locale.Builder.setUnicodeLocaleKeyword(String, String)

        Since
        - 1.8
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares this `Calendar` to the specified
        `Object`.  The result is `True` if and only if
        the argument is a `Calendar` object of the same calendar
        system that represents the same time value (millisecond offset from the
        <a href="#Epoch">Epoch</a>) under the same
        `Calendar` parameters as this object.
        
        The `Calendar` parameters are the values represented
        by the `isLenient`, `getFirstDayOfWeek`,
        `getMinimalDaysInFirstWeek` and `getTimeZone`
        methods. If there is any difference in those parameters
        between the two `Calendar`s, this method returns
        `False`.
        
        Use the .compareTo(Calendar) compareTo method to
        compare only the time values.

        Arguments
        - obj: the object to compare with.

        Returns
        - `True` if this object is equal to `obj`;
        `False` otherwise.
        """
        ...


    def hashCode(self) -> int:
        """
        Returns a hash code for this calendar.

        Returns
        - a hash code value for this object.

        Since
        - 1.2
        """
        ...


    def before(self, when: "Object") -> bool:
        """
        Returns whether this `Calendar` represents a time
        before the time represented by the specified
        `Object`. This method is equivalent to:
        ````compareTo(when) < 0````
        if and only if `when` is a `Calendar`
        instance. Otherwise, the method returns `False`.

        Arguments
        - when: the `Object` to be compared

        Returns
        - `True` if the time of this
        `Calendar` is before the time represented by
        `when`; `False` otherwise.

        See
        - .compareTo(Calendar)
        """
        ...


    def after(self, when: "Object") -> bool:
        """
        Returns whether this `Calendar` represents a time
        after the time represented by the specified
        `Object`. This method is equivalent to:
        ````compareTo(when) > 0````
        if and only if `when` is a `Calendar`
        instance. Otherwise, the method returns `False`.

        Arguments
        - when: the `Object` to be compared

        Returns
        - `True` if the time of this `Calendar` is
        after the time represented by `when`; `False`
        otherwise.

        See
        - .compareTo(Calendar)
        """
        ...


    def compareTo(self, anotherCalendar: "Calendar") -> int:
        """
        Compares the time values (millisecond offsets from the <a
        href="#Epoch">Epoch</a>) represented by two
        `Calendar` objects.

        Arguments
        - anotherCalendar: the `Calendar` to be compared.

        Returns
        - the value `0` if the time represented by the argument
        is equal to the time represented by this `Calendar`; a value
        less than `0` if the time of this `Calendar` is
        before the time represented by the argument; and a value greater than
        `0` if the time of this `Calendar` is after the
        time represented by the argument.

        Raises
        - NullPointerException: if the specified `Calendar` is
                   `null`.
        - IllegalArgumentException: if the time value of the
        specified `Calendar` object can't be obtained due to
        any invalid calendar values.

        Since
        - 1.5
        """
        ...


    def add(self, field: int, amount: int) -> None:
        """
        Adds or subtracts the specified amount of time to the given calendar field,
        based on the calendar's rules. For example, to subtract 5 days from
        the current time of the calendar, you can achieve it by calling:
        `add(Calendar.DAY_OF_MONTH, -5)`.

        Arguments
        - field: the calendar field.
        - amount: the amount of date or time to be added to the field.

        See
        - .set(int,int)
        """
        ...


    def roll(self, field: int, up: bool) -> None:
        """
        Adds or subtracts (up/down) a single unit of time on the given time
        field without changing larger fields. For example, to roll the current
        date up by one day, you can achieve it by calling:
        roll(Calendar.DATE, True).
        When rolling on the year or Calendar.YEAR field, it will roll the year
        value in the range between 1 and the value returned by calling
        `getMaximum(Calendar.YEAR)`.
        When rolling on the month or Calendar.MONTH field, other fields like
        date might conflict and, need to be changed. For instance,
        rolling the month on the date 01/31/96 will result in 02/29/96.
        When rolling on the hour-in-day or Calendar.HOUR_OF_DAY field, it will
        roll the hour value in the range between 0 and 23, which is zero-based.

        Arguments
        - field: the time field.
        - up: indicates if the value of the specified time field is to be
        rolled up or rolled down. Use True if rolling up, False otherwise.

        See
        - Calendar.set(int,int)
        """
        ...


    def roll(self, field: int, amount: int) -> None:
        """
        Adds the specified (signed) amount to the specified calendar field
        without changing larger fields.  A negative amount means to roll
        down.
        
        NOTE:  This default implementation on `Calendar` just repeatedly calls the
        version of .roll(int,boolean) roll() that rolls by one unit.  This may not
        always do the right thing.  For example, if the `DAY_OF_MONTH` field is 31,
        rolling through February will leave it set to 28.  The `GregorianCalendar`
        version of this function takes care of this problem.  Other subclasses
        should also provide overrides of this function that do the right thing.

        Arguments
        - field: the calendar field.
        - amount: the signed amount to add to the calendar `field`.

        See
        - .set(int,int)

        Since
        - 1.2
        """
        ...


    def setTimeZone(self, value: "TimeZone") -> None:
        """
        Sets the time zone with the given time zone value.

        Arguments
        - value: the given time zone.
        """
        ...


    def getTimeZone(self) -> "TimeZone":
        """
        Gets the time zone.

        Returns
        - the time zone object associated with this calendar.
        """
        ...


    def setLenient(self, lenient: bool) -> None:
        """
        Specifies whether or not date/time interpretation is to be lenient.  With
        lenient interpretation, a date such as "February 942, 1996" will be
        treated as being equivalent to the 941st day after February 1, 1996.
        With strict (non-lenient) interpretation, such dates will cause an exception to be
        thrown. The default is lenient.

        Arguments
        - lenient: `True` if the lenient mode is to be turned
        on; `False` if it is to be turned off.

        See
        - java.text.DateFormat.setLenient
        """
        ...


    def isLenient(self) -> bool:
        """
        Tells whether date/time interpretation is to be lenient.

        Returns
        - `True` if the interpretation mode of this calendar is lenient;
        `False` otherwise.

        See
        - .setLenient(boolean)
        """
        ...


    def setFirstDayOfWeek(self, value: int) -> None:
        """
        Sets what the first day of the week is; e.g., `SUNDAY` in the U.S.,
        `MONDAY` in France.

        Arguments
        - value: the given first day of the week.

        See
        - .getMinimalDaysInFirstWeek()
        """
        ...


    def getFirstDayOfWeek(self) -> int:
        """
        Gets what the first day of the week is; e.g., `SUNDAY` in the U.S.,
        `MONDAY` in France.

        Returns
        - the first day of the week.

        See
        - .getMinimalDaysInFirstWeek()
        """
        ...


    def setMinimalDaysInFirstWeek(self, value: int) -> None:
        """
        Sets what the minimal days required in the first week of the year are;
        For example, if the first week is defined as one that contains the first
        day of the first month of a year, call this method with value 1. If it
        must be a full week, use value 7.

        Arguments
        - value: the given minimal days required in the first week
        of the year.

        See
        - .getMinimalDaysInFirstWeek()
        """
        ...


    def getMinimalDaysInFirstWeek(self) -> int:
        """
        Gets what the minimal days required in the first week of the year are;
        e.g., if the first week is defined as one that contains the first day
        of the first month of a year, this method returns 1. If
        the minimal days required must be a full week, this method
        returns 7.

        Returns
        - the minimal days required in the first week of the year.

        See
        - .setMinimalDaysInFirstWeek(int)
        """
        ...


    def isWeekDateSupported(self) -> bool:
        """
        Returns whether this `Calendar` supports week dates.
        
        The default implementation of this method returns `False`.

        Returns
        - `True` if this `Calendar` supports week dates;
                `False` otherwise.

        See
        - .getWeeksInWeekYear()

        Since
        - 1.7
        """
        ...


    def getWeekYear(self) -> int:
        """
        Returns the week year represented by this `Calendar`. The
        week year is in sync with the week cycle. The .getFirstDayOfWeek() first day of the first week is the first
        day of the week year.
        
        The default implementation of this method throws an
        UnsupportedOperationException.

        Returns
        - the week year of this `Calendar`

        Raises
        - UnsupportedOperationException: if any week year numbering isn't supported
                   in this `Calendar`.

        See
        - .getMinimalDaysInFirstWeek()

        Since
        - 1.7
        """
        ...


    def setWeekDate(self, weekYear: int, weekOfYear: int, dayOfWeek: int) -> None:
        """
        Sets the date of this `Calendar` with the given date
        specifiers - week year, week of year, and day of week.
        
        Unlike the `set` method, all of the calendar fields
        and `time` values are calculated upon return.
        
        If `weekOfYear` is out of the valid week-of-year range
        in `weekYear`, the `weekYear` and `weekOfYear` values are adjusted in lenient mode, or an `IllegalArgumentException` is thrown in non-lenient mode.
        
        The default implementation of this method throws an
        `UnsupportedOperationException`.

        Arguments
        - weekYear: the week year
        - weekOfYear: the week number based on `weekYear`
        - dayOfWeek: the day of week value: one of the constants
                          for the .DAY_OF_WEEK field: .SUNDAY, ..., .SATURDAY.

        Raises
        - IllegalArgumentException: if any of the given date specifiers is invalid
                   or any of the calendar fields are inconsistent
                   with the given date specifiers in non-lenient mode
        - UnsupportedOperationException: if any week year numbering isn't supported in this
                   `Calendar`.

        See
        - .getMinimalDaysInFirstWeek()

        Since
        - 1.7
        """
        ...


    def getWeeksInWeekYear(self) -> int:
        """
        Returns the number of weeks in the week year represented by this
        `Calendar`.
        
        The default implementation of this method throws an
        `UnsupportedOperationException`.

        Returns
        - the number of weeks in the week year.

        Raises
        - UnsupportedOperationException: if any week year numbering isn't supported in this
                   `Calendar`.

        See
        - .getActualMaximum(int)

        Since
        - 1.7
        """
        ...


    def getMinimum(self, field: int) -> int:
        """
        Returns the minimum value for the given calendar field of this
        `Calendar` instance. The minimum value is defined as
        the smallest value returned by the .get(int) get method
        for any possible time value.  The minimum value depends on
        calendar system specific parameters of the instance.

        Arguments
        - field: the calendar field.

        Returns
        - the minimum value for the given calendar field.

        See
        - .getActualMaximum(int)
        """
        ...


    def getMaximum(self, field: int) -> int:
        """
        Returns the maximum value for the given calendar field of this
        `Calendar` instance. The maximum value is defined as
        the largest value returned by the .get(int) get method
        for any possible time value. The maximum value depends on
        calendar system specific parameters of the instance.

        Arguments
        - field: the calendar field.

        Returns
        - the maximum value for the given calendar field.

        See
        - .getActualMaximum(int)
        """
        ...


    def getGreatestMinimum(self, field: int) -> int:
        """
        Returns the highest minimum value for the given calendar field
        of this `Calendar` instance. The highest minimum
        value is defined as the largest value returned by .getActualMinimum(int) for any possible time value. The
        greatest minimum value depends on calendar system specific
        parameters of the instance.

        Arguments
        - field: the calendar field.

        Returns
        - the highest minimum value for the given calendar field.

        See
        - .getActualMaximum(int)
        """
        ...


    def getLeastMaximum(self, field: int) -> int:
        """
        Returns the lowest maximum value for the given calendar field
        of this `Calendar` instance. The lowest maximum
        value is defined as the smallest value returned by .getActualMaximum(int) for any possible time value. The least
        maximum value depends on calendar system specific parameters of
        the instance. For example, a `Calendar` for the
        Gregorian calendar system returns 28 for the
        `DAY_OF_MONTH` field, because the 28th is the last
        day of the shortest month of this calendar, February in a
        common year.

        Arguments
        - field: the calendar field.

        Returns
        - the lowest maximum value for the given calendar field.

        See
        - .getActualMaximum(int)
        """
        ...


    def getActualMinimum(self, field: int) -> int:
        """
        Returns the minimum value that the specified calendar field
        could have, given the time value of this `Calendar`.
        
        The default implementation of this method uses an iterative
        algorithm to determine the actual minimum value for the
        calendar field. Subclasses should, if possible, override this
        with a more efficient implementation - in many cases, they can
        simply return `getMinimum()`.

        Arguments
        - field: the calendar field

        Returns
        - the minimum of the given calendar field for the time
        value of this `Calendar`

        See
        - .getActualMaximum(int)

        Since
        - 1.2
        """
        ...


    def getActualMaximum(self, field: int) -> int:
        """
        Returns the maximum value that the specified calendar field
        could have, given the time value of this
        `Calendar`. For example, the actual maximum value of
        the `MONTH` field is 12 in some years, and 13 in
        other years in the Hebrew calendar system.
        
        The default implementation of this method uses an iterative
        algorithm to determine the actual maximum value for the
        calendar field. Subclasses should, if possible, override this
        with a more efficient implementation.

        Arguments
        - field: the calendar field

        Returns
        - the maximum of the given calendar field for the time
        value of this `Calendar`

        See
        - .getActualMinimum(int)

        Since
        - 1.2
        """
        ...


    def clone(self) -> "Object":
        """
        Creates and returns a copy of this object.

        Returns
        - a copy of this object.
        """
        ...


    def toString(self) -> str:
        """
        Return a string representation of this calendar. This method
        is intended to be used only for debugging purposes, and the
        format of the returned string may vary between implementations.
        The returned string may be empty but may not be `null`.

        Returns
        - a string representation of this calendar.
        """
        ...


    def toInstant(self) -> "Instant":
        """
        Converts this object to an Instant.
        
        The conversion creates an `Instant` that represents the
        same point on the time-line as this `Calendar`.

        Returns
        - the instant representing the same point on the time-line

        Since
        - 1.8
        """
        ...


    class Builder:
        """
        `Calendar.Builder` is used for creating a `Calendar` from
        various date-time parameters.
        
        There are two ways to set a `Calendar` to a date-time value. One
        is to set the instant parameter to a millisecond offset from the <a
        href="Calendar.html#Epoch">Epoch</a>. The other is to set individual
        field parameters, such as Calendar.YEAR YEAR, to their desired
        values. These two ways can't be mixed. Trying to set both the instant and
        individual fields will cause an IllegalStateException to be
        thrown. However, it is permitted to override previous values of the
        instant or field parameters.
        
        If no enough field parameters are given for determining date and/or
        time, calendar specific default values are used when building a
        `Calendar`. For example, if the Calendar.YEAR YEAR value
        isn't given for the Gregorian calendar, 1970 will be used. If there are
        any conflicts among field parameters, the <a
        href="Calendar.html#resolution"> resolution rules</a> are applied.
        Therefore, the order of field setting matters.
        
        In addition to the date-time parameters,
        the .setLocale(Locale) locale,
        .setTimeZone(TimeZone) time zone,
        .setWeekDefinition(int, int) week definition, and
        .setLenient(boolean) leniency mode parameters can be set.
        
        **Examples**
        The following are sample usages. Sample code assumes that the
        `Calendar` constants are statically imported.
        
        The following code produces a `Calendar` with date 2012-12-31
        (Gregorian) because Monday is the first day of a week with the <a
        href="GregorianCalendar.html#iso8601_compatible_setting"> ISO 8601
        compatible week parameters</a>.
        ```
          Calendar cal = new Calendar.Builder().setCalendarType("iso8601")
                               .setWeekDate(2013, 1, MONDAY).build();```
        The following code produces a Japanese `Calendar` with date
        1989-01-08 (Gregorian), assuming that the default Calendar.ERA ERA
        is *Heisei* that started on that day.
        ```
          Calendar cal = new Calendar.Builder().setCalendarType("japanese")
                               .setFields(YEAR, 1, DAY_OF_YEAR, 1).build();```

        See
        - Calendar.fields

        Since
        - 1.8
        """

        def __init__(self):
            """
            Constructs a `Calendar.Builder`.
            """
            ...


        def setInstant(self, instant: int) -> "Builder":
            """
            Sets the instant parameter to the given `instant` value that is
            a millisecond offset from <a href="Calendar.html#Epoch">the
            Epoch</a>.

            Arguments
            - instant: a millisecond offset from the Epoch

            Returns
            - this `Calendar.Builder`

            Raises
            - IllegalStateException: if any of the field parameters have
                                          already been set

            See
            - Calendar.time
            """
            ...


        def setInstant(self, instant: "Date") -> "Builder":
            """
            Sets the instant parameter to the `instant` value given by a
            Date. This method is equivalent to a call to
            .setInstant(long) setInstant(instant.getTime()).

            Arguments
            - instant: a `Date` representing a millisecond offset from
                           the Epoch

            Returns
            - this `Calendar.Builder`

            Raises
            - NullPointerException: if `instant` is `null`
            - IllegalStateException: if any of the field parameters have
                                          already been set

            See
            - Calendar.time
            """
            ...


        def set(self, field: int, value: int) -> "Builder":
            """
            Sets the `field` parameter to the given `value`.
            `field` is an index to the Calendar.fields, such as
            Calendar.DAY_OF_MONTH DAY_OF_MONTH. Field value validation is
            not performed in this method. Any out of range values are either
            normalized in lenient mode or detected as an invalid value in
            non-lenient mode when building a `Calendar`.

            Arguments
            - field: an index to the `Calendar` fields
            - value: the field value

            Returns
            - this `Calendar.Builder`

            Raises
            - IllegalArgumentException: if `field` is invalid
            - IllegalStateException: if the instant value has already been set,
                                 or if fields have been set too many
                                 (approximately Integer.MAX_VALUE) times.

            See
            - Calendar.set(int, int)
            """
            ...


        def setFields(self, *fieldValuePairs: Tuple[int, ...]) -> "Builder":
            """
            Sets field parameters to their values given by
            `fieldValuePairs` that are pairs of a field and its value.
            For example,
            ```
              setFields(Calendar.YEAR, 2013,
                        Calendar.MONTH, Calendar.DECEMBER,
                        Calendar.DAY_OF_MONTH, 23);```
            is equivalent to the sequence of the following
            .set(int, int) set calls:
            ```
              set(Calendar.YEAR, 2013)
              .set(Calendar.MONTH, Calendar.DECEMBER)
              .set(Calendar.DAY_OF_MONTH, 23);```

            Arguments
            - fieldValuePairs: field-value pairs

            Returns
            - this `Calendar.Builder`

            Raises
            - NullPointerException: if `fieldValuePairs` is `null`
            - IllegalArgumentException: if any of fields are invalid,
                        or if `fieldValuePairs.length` is an odd number.
            - IllegalStateException: if the instant value has been set,
                        or if fields have been set too many (approximately
                        Integer.MAX_VALUE) times.
            """
            ...


        def setDate(self, year: int, month: int, dayOfMonth: int) -> "Builder":
            """
            Sets the date field parameters to the values given by `year`,
            `month`, and `dayOfMonth`. This method is equivalent to
            a call to:
            ```
              setFields(Calendar.YEAR, year,
                        Calendar.MONTH, month,
                        Calendar.DAY_OF_MONTH, dayOfMonth);```

            Arguments
            - year: the Calendar.YEAR YEAR value
            - month: the Calendar.MONTH MONTH value
                              (the month numbering is *0-based*).
            - dayOfMonth: the Calendar.DAY_OF_MONTH DAY_OF_MONTH value

            Returns
            - this `Calendar.Builder`
            """
            ...


        def setTimeOfDay(self, hourOfDay: int, minute: int, second: int) -> "Builder":
            """
            Sets the time of day field parameters to the values given by
            `hourOfDay`, `minute`, and `second`. This method is
            equivalent to a call to:
            ```
              setTimeOfDay(hourOfDay, minute, second, 0);```

            Arguments
            - hourOfDay: the Calendar.HOUR_OF_DAY HOUR_OF_DAY value
                             (24-hour clock)
            - minute: the Calendar.MINUTE MINUTE value
            - second: the Calendar.SECOND SECOND value

            Returns
            - this `Calendar.Builder`
            """
            ...


        def setTimeOfDay(self, hourOfDay: int, minute: int, second: int, millis: int) -> "Builder":
            """
            Sets the time of day field parameters to the values given by
            `hourOfDay`, `minute`, `second`, and
            `millis`. This method is equivalent to a call to:
            ```
              setFields(Calendar.HOUR_OF_DAY, hourOfDay,
                        Calendar.MINUTE, minute,
                        Calendar.SECOND, second,
                        Calendar.MILLISECOND, millis);```

            Arguments
            - hourOfDay: the Calendar.HOUR_OF_DAY HOUR_OF_DAY value
                             (24-hour clock)
            - minute: the Calendar.MINUTE MINUTE value
            - second: the Calendar.SECOND SECOND value
            - millis: the Calendar.MILLISECOND MILLISECOND value

            Returns
            - this `Calendar.Builder`
            """
            ...


        def setWeekDate(self, weekYear: int, weekOfYear: int, dayOfWeek: int) -> "Builder":
            """
            Sets the week-based date parameters to the values with the given
            date specifiers - week year, week of year, and day of week.
            
            If the specified calendar doesn't support week dates, the
            .build() build method will throw an IllegalArgumentException.

            Arguments
            - weekYear: the week year
            - weekOfYear: the week number based on `weekYear`
            - dayOfWeek: the day of week value: one of the constants
                for the Calendar.DAY_OF_WEEK DAY_OF_WEEK field:
                Calendar.SUNDAY SUNDAY, ..., Calendar.SATURDAY SATURDAY.

            Returns
            - this `Calendar.Builder`

            See
            - Calendar.isWeekDateSupported()
            """
            ...


        def setTimeZone(self, zone: "TimeZone") -> "Builder":
            """
            Sets the time zone parameter to the given `zone`. If no time
            zone parameter is given to this `Calendar.Builder`, the
            TimeZone.getDefault() default
            {@code TimeZone} will be used in the .build() build
            method.

            Arguments
            - zone: the TimeZone

            Returns
            - this `Calendar.Builder`

            Raises
            - NullPointerException: if `zone` is `null`

            See
            - Calendar.setTimeZone(TimeZone)
            """
            ...


        def setLenient(self, lenient: bool) -> "Builder":
            """
            Sets the lenient mode parameter to the value given by `lenient`.
            If no lenient parameter is given to this `Calendar.Builder`,
            lenient mode will be used in the .build() build method.

            Arguments
            - lenient: `True` for lenient mode;
                           `False` for non-lenient mode

            Returns
            - this `Calendar.Builder`

            See
            - Calendar.setLenient(boolean)
            """
            ...


        def setCalendarType(self, type: str) -> "Builder":
            """
            Sets the calendar type parameter to the given `type`. The
            calendar type given by this method has precedence over any explicit
            or implicit calendar type given by the
            .setLocale(Locale) locale.
            
            In addition to the available calendar types returned by the
            Calendar.getAvailableCalendarTypes() Calendar.getAvailableCalendarTypes
            method, `"gregorian"` and `"iso8601"` as aliases of
            `"gregory"` can be used with this method.

            Arguments
            - type: the calendar type

            Returns
            - this `Calendar.Builder`

            Raises
            - NullPointerException: if `type` is `null`
            - IllegalArgumentException: if `type` is unknown
            - IllegalStateException: if another calendar type has already been set

            See
            - Calendar.getAvailableCalendarTypes()
            """
            ...


        def setLocale(self, locale: "Locale") -> "Builder":
            """
            Sets the locale parameter to the given `locale`. If no locale
            is given to this `Calendar.Builder`, the Locale.getDefault(Locale.Category) default {@code Locale}
            for Locale.Category.FORMAT will be used.
            
            If no calendar type is explicitly given by a call to the
            .setCalendarType(String) setCalendarType method,
            the `Locale` value is used to determine what type of
            `Calendar` to be built.
            
            If no week definition parameters are explicitly given by a call to
            the .setWeekDefinition(int,int) setWeekDefinition method, the
            `Locale`'s default values are used.

            Arguments
            - locale: the Locale

            Returns
            - this `Calendar.Builder`

            Raises
            - NullPointerException: if `locale` is `null`

            See
            - Calendar.getInstance(Locale)
            """
            ...


        def setWeekDefinition(self, firstDayOfWeek: int, minimalDaysInFirstWeek: int) -> "Builder":
            """
            Sets the week definition parameters to the values given by
            `firstDayOfWeek` and `minimalDaysInFirstWeek` that are
            used to determine the <a href="Calendar.html#first_week">first
            week</a> of a year. The parameters given by this method have
            precedence over the default values given by the
            .setLocale(Locale) locale.

            Arguments
            - firstDayOfWeek: the first day of a week; one of
                                  Calendar.SUNDAY to Calendar.SATURDAY
            - minimalDaysInFirstWeek: the minimal number of days in the first
                                          week (1..7)

            Returns
            - this `Calendar.Builder`

            Raises
            - IllegalArgumentException: if `firstDayOfWeek` or
                                             `minimalDaysInFirstWeek` is invalid

            See
            - Calendar.getMinimalDaysInFirstWeek()
            """
            ...


        def build(self) -> "Calendar":
            """
            Returns a `Calendar` built from the parameters set by the
            setter methods. The calendar type given by the .setCalendarType(String)
            setCalendarType method or the .setLocale(Locale) locale is
            used to determine what `Calendar` to be created. If no explicit
            calendar type is given, the locale's default calendar is created.
            
            If the calendar type is `"iso8601"`, the
            GregorianCalendar.setGregorianChange(Date) Gregorian change date
            of a GregorianCalendar is set to `Date(Long.MIN_VALUE)`
            to be the *proleptic* Gregorian calendar. Its week definition
            parameters are also set to be <a
            href="GregorianCalendar.html#iso8601_compatible_setting">compatible
            with the ISO 8601 standard</a>. Note that the
            GregorianCalendar.getCalendarType() getCalendarType method of
            a `GregorianCalendar` created with `"iso8601"` returns
            `"gregory"`.
            
            The default values are used for locale and time zone if these
            parameters haven't been given explicitly.
            
            If the locale contains the time zone with "tz"
            <a href="Locale.html#def_locale_extension">Unicode extension</a>,
            and time zone hasn't been given explicitly, time zone in the locale
            is used.
            
            Any out of range field values are either normalized in lenient
            mode or detected as an invalid value in non-lenient mode.

            Returns
            - a `Calendar` built with parameters of this `Calendar.Builder`

            Raises
            - IllegalArgumentException: if the calendar type is unknown, or
                        if any invalid field values are given in non-lenient mode, or
                        if a week date is given for the calendar type that doesn't
                        support week dates.

            See
            - TimeZone.getDefault()
            """
            ...



"""
Python module generated from Java source file org.bukkit.entity.Hanging

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import BlockFace
from org.bukkit.entity import *
from org.bukkit.material import Attachable
from typing import Any, Callable, Iterable, Tuple


class Hanging(Entity, Attachable):
    """
    Represents a Hanging entity
    """

    def setFacingDirection(self, face: "BlockFace", force: bool) -> bool:
        """
        Sets the direction of the hanging entity, potentially overriding rules
        of placement. Note that if the result is not valid the object would
        normally drop as an item.

        Arguments
        - face: The new direction.
        - force: Whether to force it.

        Returns
        - False if force was False and there was no block for it to
            attach to in order to face the given direction.
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.TNTPrimed

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Location
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class TNTPrimed(Explosive):
    """
    Represents a Primed TNT.
    """

    def setFuseTicks(self, fuseTicks: int) -> None:
        """
        Set the number of ticks until the TNT blows up after being primed.

        Arguments
        - fuseTicks: The fuse ticks
        """
        ...


    def getFuseTicks(self) -> int:
        """
        Retrieve the number of ticks until the explosion of this TNTPrimed
        entity

        Returns
        - the number of ticks until this TNTPrimed explodes
        """
        ...


    def getSource(self) -> "Entity":
        """
        Gets the source of this primed TNT. The source is the entity
        responsible for the creation of this primed TNT. (I.E. player ignites
        TNT with flint and steel.) Take note that this can be null if there is
        no suitable source. (created by the org.bukkit.World.spawn(Location, Class) method, for example.)
        
        The source will become null if the chunk this primed TNT is in is
        unloaded then reloaded. The source entity may be invalid if for example
        it has since died or been unloaded. Callers should check
        Entity.isValid().

        Returns
        - the source of this primed TNT
        """
        ...


    def setSource(self, source: "Entity") -> None:
        """
        Sets the source of this primed TNT.
        
        The source is the entity responsible for the creation of this primed TNT.
        
        Must be instance of org.bukkit.entity.LivingEntity otherwise will
        be set to null. The parameter is typed org.bukkit.entity.Entity to be consistent with org.bukkit.entity.TNTPrimed.getSource() method.

        Arguments
        - source: the source of this primed TNT
        """
        ...



"""
Python module generated from Java source file com.google.common.primitives.Floats

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.base import Converter
from com.google.common.primitives import *
from com.google.errorprone.annotations import InlineMe
from java.io import Serializable
from java.util import AbstractList
from java.util import Arrays
from java.util import Collections
from java.util import Comparator
from java.util import RandomAccess
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class Floats(FloatsMethodsForWeb):
    """
    Static utility methods pertaining to `float` primitives, that are not already found in
    either Float or Arrays.
    
    See the Guava User Guide article on <a
    href="https://github.com/google/guava/wiki/PrimitivesExplained">primitive utilities</a>.

    Author(s)
    - Kevin Bourrillion

    Since
    - 1.0
    """

    BYTES = Float.SIZE / Byte.SIZE
    """
    The number of bytes required to represent a primitive `float` value.
    
    **Java 8+ users:** use Float.BYTES instead.

    Since
    - 10.0
    """


    @staticmethod
    def hashCode(value: float) -> int:
        """
        Returns a hash code for `value`; equal to the result of invoking `((Float)
        value).hashCode()`.
        
        **Java 8+ users:** use Float.hashCode(float) instead.

        Arguments
        - value: a primitive `float` value

        Returns
        - a hash code for the value
        """
        ...


    @staticmethod
    def compare(a: float, b: float) -> int:
        """
        Compares the two specified `float` values using Float.compare(float, float). You
        may prefer to invoke that method directly; this method exists only for consistency with the
        other utilities in this package.
        
        **Note:** this method simply delegates to the JDK method Float.compare. It is
        provided for consistency with the other primitive types, whose compare methods were not added
        to the JDK until JDK 7.

        Arguments
        - a: the first `float` to compare
        - b: the second `float` to compare

        Returns
        - the result of invoking Float.compare(float, float)
        """
        ...


    @staticmethod
    def isFinite(value: float) -> bool:
        """
        Returns `True` if `value` represents a real number. This is equivalent to, but not
        necessarily implemented as, `!(Float.isInfinite(value) || Float.isNaN(value))`.
        
        **Java 8+ users:** use Float.isFinite(float) instead.

        Since
        - 10.0
        """
        ...


    @staticmethod
    def contains(array: list[float], target: float) -> bool:
        """
        Returns `True` if `target` is present as an element anywhere in `array`. Note
        that this always returns `False` when `target` is `NaN`.

        Arguments
        - array: an array of `float` values, possibly empty
        - target: a primitive `float` value

        Returns
        - `True` if `array[i] == target` for some value of `i`
        """
        ...


    @staticmethod
    def indexOf(array: list[float], target: float) -> int:
        """
        Returns the index of the first appearance of the value `target` in `array`. Note
        that this always returns `-1` when `target` is `NaN`.

        Arguments
        - array: an array of `float` values, possibly empty
        - target: a primitive `float` value

        Returns
        - the least index `i` for which `array[i] == target`, or `-1` if no
            such index exists.
        """
        ...


    @staticmethod
    def indexOf(array: list[float], target: list[float]) -> int:
        """
        Returns the start position of the first occurrence of the specified `target` within
        `array`, or `-1` if there is no such occurrence.
        
        More formally, returns the lowest index `i` such that `Arrays.copyOfRange(array,
        i, i + target.length)` contains exactly the same elements as `target`.
        
        Note that this always returns `-1` when `target` contains `NaN`.

        Arguments
        - array: the array to search for the sequence `target`
        - target: the array to search for as a sub-sequence of `array`
        """
        ...


    @staticmethod
    def lastIndexOf(array: list[float], target: float) -> int:
        """
        Returns the index of the last appearance of the value `target` in `array`. Note
        that this always returns `-1` when `target` is `NaN`.

        Arguments
        - array: an array of `float` values, possibly empty
        - target: a primitive `float` value

        Returns
        - the greatest index `i` for which `array[i] == target`, or `-1` if no
            such index exists.
        """
        ...


    @staticmethod
    def min(*array: Tuple[float, ...]) -> float:
        """
        Returns the least value present in `array`, using the same rules of comparison as Math.min(float, float).

        Arguments
        - array: a *nonempty* array of `float` values

        Returns
        - the value present in `array` that is less than or equal to every other value in
            the array

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def max(*array: Tuple[float, ...]) -> float:
        """
        Returns the greatest value present in `array`, using the same rules of comparison as
        Math.max(float, float).

        Arguments
        - array: a *nonempty* array of `float` values

        Returns
        - the value present in `array` that is greater than or equal to every other value
            in the array

        Raises
        - IllegalArgumentException: if `array` is empty
        """
        ...


    @staticmethod
    def constrainToRange(value: float, min: float, max: float) -> float:
        """
        Returns the value nearest to `value` which is within the closed range `[min..max]`.
        
        If `value` is within the range `[min..max]`, `value` is returned
        unchanged. If `value` is less than `min`, `min` is returned, and if `value` is greater than `max`, `max` is returned.

        Arguments
        - value: the `float` value to constrain
        - min: the lower bound (inclusive) of the range to constrain `value` to
        - max: the upper bound (inclusive) of the range to constrain `value` to

        Raises
        - IllegalArgumentException: if `min > max`

        Since
        - 21.0
        """
        ...


    @staticmethod
    def concat(*arrays: Tuple[list[float], ...]) -> list[float]:
        """
        Returns the values from each provided array combined into a single array. For example, `concat(new float[] {a, b`, new float[] {}, new float[] {c}} returns the array `{a, b,
        c`}.

        Arguments
        - arrays: zero or more `float` arrays

        Returns
        - a single array containing all the values from the source arrays, in order

        Raises
        - IllegalArgumentException: if the total number of elements in `arrays` does not fit
            in an `int`
        """
        ...


    @staticmethod
    def stringConverter() -> "Converter"[str, "Float"]:
        """
        Returns a serializable converter object that converts between strings and floats using Float.valueOf and Float.toString().

        Since
        - 16.0
        """
        ...


    @staticmethod
    def ensureCapacity(array: list[float], minLength: int, padding: int) -> list[float]:
        """
        Returns an array containing the same values as `array`, but guaranteed to be of a
        specified minimum length. If `array` already has a length of at least `minLength`,
        it is returned directly. Otherwise, a new array of size `minLength + padding` is
        returned, containing the values of `array`, and zeroes in the remaining places.

        Arguments
        - array: the source array
        - minLength: the minimum length the returned array must guarantee
        - padding: an extra amount to "grow" the array by if growth is necessary

        Returns
        - an array containing the values of `array`, with guaranteed minimum length `minLength`

        Raises
        - IllegalArgumentException: if `minLength` or `padding` is negative
        """
        ...


    @staticmethod
    def join(separator: str, *array: Tuple[float, ...]) -> str:
        """
        Returns a string containing the supplied `float` values, converted to strings as
        specified by Float.toString(float), and separated by `separator`. For example,
        `join("-", 1.0f, 2.0f, 3.0f)` returns the string `"1.0-2.0-3.0"`.
        
        Note that Float.toString(float) formats `float` differently in GWT. In the
        previous example, it returns the string `"1-2-3"`.

        Arguments
        - separator: the text that should appear between consecutive values in the resulting string
            (but not at the start or end)
        - array: an array of `float` values, possibly empty
        """
        ...


    @staticmethod
    def lexicographicalComparator() -> "Comparator"[list[float]]:
        """
        Returns a comparator that compares two `float` arrays <a
        href="http://en.wikipedia.org/wiki/Lexicographical_order">lexicographically</a>. That is, it
        compares, using .compare(float, float)), the first pair of values that follow any
        common prefix, or when one array is a prefix of the other, treats the shorter array as the
        lesser. For example, `[] < [1.0f] < [1.0f, 2.0f] < [2.0f]`.
        
        The returned comparator is inconsistent with Object.equals(Object) (since arrays
        support only identity equality), but it is consistent with Arrays.equals(float[],
        float[]).

        Since
        - 2.0
        """
        ...


    @staticmethod
    def sortDescending(array: list[float]) -> None:
        """
        Sorts the elements of `array` in descending order.
        
        Note that this method uses the total order imposed by Float.compare, which treats
        all NaN values as equal and 0.0 as greater than -0.0.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def sortDescending(array: list[float], fromIndex: int, toIndex: int) -> None:
        """
        Sorts the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive in descending order.
        
        Note that this method uses the total order imposed by Float.compare, which treats
        all NaN values as equal and 0.0 as greater than -0.0.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def reverse(array: list[float]) -> None:
        """
        Reverses the elements of `array`. This is equivalent to `Collections.reverse(Floats.asList(array))`, but is likely to be more efficient.

        Since
        - 23.1
        """
        ...


    @staticmethod
    def reverse(array: list[float], fromIndex: int, toIndex: int) -> None:
        """
        Reverses the elements of `array` between `fromIndex` inclusive and `toIndex`
        exclusive. This is equivalent to `Collections.reverse(Floats.asList(array).subList(fromIndex, toIndex))`, but is likely to be
        more efficient.

        Raises
        - IndexOutOfBoundsException: if `fromIndex < 0`, `toIndex > array.length`, or
            `toIndex > fromIndex`

        Since
        - 23.1
        """
        ...


    @staticmethod
    def rotate(array: list[float], distance: int) -> None:
        """
        Performs a right rotation of `array` of "distance" places, so that the first element is
        moved to index "distance", and the element at index `i` ends up at index `(distance
        + i) mod array.length`. This is equivalent to `Collections.rotate(Floats.asList(array),
        distance)`, but is considerably faster and avoids allocation and garbage collection.
        
        The provided "distance" may be negative, which will rotate left.

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def rotate(array: list[float], distance: int, fromIndex: int, toIndex: int) -> None:
        """
        Performs a right rotation of `array` between `fromIndex` inclusive and `toIndex` exclusive. This is equivalent to `Collections.rotate(Floats.asList(array).subList(fromIndex, toIndex), distance)`, but is
        considerably faster and avoids allocations and garbage collection.
        
        The provided "distance" may be negative, which will rotate left.

        Raises
        - IndexOutOfBoundsException: if `fromIndex < 0`, `toIndex > array.length`, or
            `toIndex > fromIndex`

        Since
        - 32.0.0
        """
        ...


    @staticmethod
    def toArray(collection: Iterable["Number"]) -> list[float]:
        """
        Returns an array containing each value of `collection`, converted to a `float`
        value in the manner of Number.floatValue.
        
        Elements are copied from the argument collection as if by `collection.toArray()`.
        Calling this method is as thread-safe as calling that method.

        Arguments
        - collection: a collection of `Number` instances

        Returns
        - an array containing the same values as `collection`, in the same order, converted
            to primitives

        Raises
        - NullPointerException: if `collection` or any of its elements is null

        Since
        - 1.0 (parameter was `Collection<Float>` before 12.0)
        """
        ...


    @staticmethod
    def asList(*backingArray: Tuple[float, ...]) -> list["Float"]:
        """
        Returns a fixed-size list backed by the specified array, similar to Arrays.asList(Object[]). The list supports List.set(int, Object), but any attempt to
        set a value to `null` will result in a NullPointerException.
        
        The returned list maintains the values, but not the identities, of `Float` objects
        written to or read from it. For example, whether `list.get(0) == list.get(0)` is True for
        the returned list is unspecified.
        
        The returned list may have unexpected behavior if it contains `NaN`, or if `NaN`
        is used as a parameter to any of its methods.
        
        The returned list is serializable.

        Arguments
        - backingArray: the array to back the list

        Returns
        - a list view of the array
        """
        ...


    @staticmethod
    def tryParse(string: str) -> "Float":
        """
        Parses the specified string as a single-precision floating point value. The ASCII character
        `'-'` (`'&#92;u002D'`) is recognized as the minus sign.
        
        Unlike Float.parseFloat(String), this method returns `null` instead of
        throwing an exception if parsing fails. Valid inputs are exactly those accepted by Float.valueOf(String), except that leading and trailing whitespace is not permitted.
        
        This implementation is likely to be faster than `Float.parseFloat` if many failures
        are expected.

        Arguments
        - string: the string representation of a `float` value

        Returns
        - the floating point value represented by `string`, or `null` if `string` has a length of zero or cannot be parsed as a `float` value

        Raises
        - NullPointerException: if `string` is `null`

        Since
        - 14.0
        """
        ...



"""
Python module generated from Java source file java.net.URL

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import File
from java.io import IOException
from java.io import InputStream
from java.io import InvalidObjectException
from java.io.ObjectInputStream import GetField
from java.io import ObjectStreamException
from java.io import ObjectStreamField
from java.net import *
from java.net.spi import URLStreamHandlerProvider
from java.nio.file import Path
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import Hashtable
from java.util import Iterator
from java.util import Locale
from java.util import NoSuchElementException
from java.util import ServiceConfigurationError
from java.util import ServiceLoader
from jdk.internal.access import JavaNetURLAccess
from jdk.internal.access import SharedSecrets
from jdk.internal.misc import VM
from sun.net.util import IPAddressUtil
from sun.security.action import GetPropertyAction
from sun.security.util import SecurityConstants
from typing import Any, Callable, Iterable, Tuple


class URL(Serializable):
    """
    Class `URL` represents a Uniform Resource
    Locator, a pointer to a "resource" on the World
    Wide Web. A resource can be something as simple as a file or a
    directory, or it can be a reference to a more complicated object,
    such as a query to a database or to a search engine. More
    information on the types of URLs and their formats can be found at:
    <a href=
    "http://web.archive.org/web/20051219043731/http://archive.ncsa.uiuc.edu/SDG/Software/Mosaic/Demo/url-primer.html">
    *Types of URL*</a>
    
    In general, a URL can be broken into several parts. Consider the
    following example:
    <blockquote>```
        http://www.example.com/docs/resource1.html
    ```</blockquote>
    
    The URL above indicates that the protocol to use is
    `http` (HyperText Transfer Protocol) and that the
    information resides on a host machine named
    `www.example.com`. The information on that host
    machine is named `/docs/resource1.html`. The exact
    meaning of this name on the host machine is both protocol
    dependent and host dependent. The information normally resides in
    a file, but it could be generated on the fly. This component of
    the URL is called the *path* component.
    
    A URL can optionally specify a "port", which is the
    port number to which the TCP connection is made on the remote host
    machine. If the port is not specified, the default port for
    the protocol is used instead. For example, the default port for
    `http` is `80`. An alternative port could be
    specified as:
    <blockquote>```
        http://www.example.com:1080/docs/resource1.html
    ```</blockquote>
    
    The syntax of `URL` is defined by  <a
    href="http://www.ietf.org/rfc/rfc2396.txt">*RFC&nbsp;2396: Uniform
    Resource Identifiers (URI): Generic Syntax*</a>, amended by <a
    href="http://www.ietf.org/rfc/rfc2732.txt">*RFC&nbsp;2732: Format for
    Literal IPv6 Addresses in URLs*</a>. The Literal IPv6 address format
    also supports scope_ids. The syntax and usage of scope_ids is described
    <a href="Inet6Address.html#scoped">here</a>.
    
    A URL may have appended to it a "fragment", also known
    as a "ref" or a "reference". The fragment is indicated by the sharp
    sign character "#" followed by more characters. For example,
    <blockquote>```
        http://www.example.com/index.html#chapter1
    ```</blockquote>
    
    This fragment is not technically part of the URL. Rather, it
    indicates that after the specified resource is retrieved, the
    application is specifically interested in that part of the
    document that has the tag `chapter1` attached to it. The
    meaning of a tag is resource specific.
    
    An application can also specify a "relative URL",
    which contains only enough information to reach the resource
    relative to another URL. Relative URLs are frequently used within
    HTML pages. For example, if the contents of the URL:
    <blockquote>```
        http://www.example.com/index.html
    ```</blockquote>
    contained within it the relative URL:
    <blockquote>```
        FAQ.html
    ```</blockquote>
    it would be a shorthand for:
    <blockquote>```
        http://www.example.com/FAQ.html
    ```</blockquote>
    
    The relative URL need not specify all the components of a URL. If
    the protocol, host name, or port number is missing, the value is
    inherited from the fully specified URL. The file component must be
    specified. The optional fragment is not inherited.
    
    The URL class does not itself encode or decode any URL components
    according to the escaping mechanism defined in RFC2396. It is the
    responsibility of the caller to encode any fields, which need to be
    escaped prior to calling URL, and also to decode any escaped fields,
    that are returned from URL. Furthermore, because URL has no knowledge
    of URL escaping, it does not recognise equivalence between the encoded
    or decoded form of the same URL. For example, the two URLs:
    ```    http://foo.com/hello world/ and http://foo.com/hello%20world```
    would be considered not equal to each other.
    
    Note, the java.net.URI class does perform escaping of its
    component fields in certain circumstances. The recommended way
    to manage the encoding and decoding of URLs is to use java.net.URI,
    and to convert between these two classes using .toURI() and
    URI.toURL().
    
    The URLEncoder and URLDecoder classes can also be
    used, but only for HTML form encoding, which is not the same
    as the encoding scheme defined in RFC2396.

    Author(s)
    - James Gosling

    Since
    - 1.0

    Unknown Tags
    - Applications working with file paths and file URIs should take great
    care to use the appropriate methods to convert between the two.
    The Path.of(URI) factory method and the File.File(URI)
    constructor can be used to create Path or File
    objects from a file URI. Path.toUri() and File.toURI()
    can be used to create a URI from a file path, which can be
    converted to URL using URI.toURL().
    Applications should never try to .URL(String, String, String)
    construct or .URL(String) parse a `URL`
    from the direct string representation of a `File` or `Path`
    instance.
    
    Some components of a URL or URI, such as *userinfo*, may
    be abused to construct misleading URLs or URIs. Applications
    that deal with URLs or URIs should take into account
    the recommendations advised in <a
    href="https://tools.ietf.org/html/rfc3986#section-7">RFC3986,
    Section 7, Security Considerations</a>.
    """

    def __init__(self, protocol: str, host: str, port: int, file: str):
        """
        Creates a `URL` object from the specified
        `protocol`, `host`, `port`
        number, and `file`.
        
        `host` can be expressed as a host name or a literal
        IP address. If IPv6 literal address is used, it should be
        enclosed in square brackets (`'['` and `']'`), as
        specified by <a
        href="http://www.ietf.org/rfc/rfc2732.txt">RFC&nbsp;2732</a>;
        However, the literal IPv6 address format defined in <a
        href="http://www.ietf.org/rfc/rfc2373.txt">*RFC&nbsp;2373: IP
        Version 6 Addressing Architecture*</a> is also accepted.
        
        Specifying a `port` number of `-1`
        indicates that the URL should use the default port for the
        protocol.
        
        If this is the first URL object being created with the specified
        protocol, a *stream protocol handler* object, an instance of
        class `URLStreamHandler`, is created for that protocol:
        <ol>
        - If the application has previously set up an instance of
            `URLStreamHandlerFactory` as the stream handler factory,
            then the `createURLStreamHandler` method of that instance
            is called with the protocol string as an argument to create the
            stream protocol handler.
        - If no `URLStreamHandlerFactory` has yet been set up,
            or if the factory's `createURLStreamHandler` method
            returns `null`, then the java.util.ServiceLoader
            ServiceLoader mechanism is used to locate java.net.spi.URLStreamHandlerProvider URLStreamHandlerProvider
            implementations using the system class
            loader. The order that providers are located is implementation
            specific, and an implementation is free to cache the located
            providers. A java.util.ServiceConfigurationError
            ServiceConfigurationError, `Error` or `RuntimeException`
            thrown from the `createURLStreamHandler`, if encountered, will
            be propagated to the calling thread. The `createURLStreamHandler` method of each provider, if instantiated, is
            invoked, with the protocol string, until a provider returns non-null,
            or all providers have been exhausted.
        - If the previous step fails to find a protocol handler, the
            constructor reads the value of the system property:
            <blockquote>java.protocol.handler.pkgs</blockquote>
            If the value of that system property is not `null`,
            it is interpreted as a list of packages separated by a vertical
            slash character '`|`'. The constructor tries to load
            the class named:
            <blockquote>`<package>.<protocol>.Handler`</blockquote>
            where `<package>` is replaced by the name of the package
            and `<protocol>` is replaced by the name of the protocol.
            If this class does not exist, or if the class exists but it is not
            a subclass of `URLStreamHandler`, then the next package
            in the list is tried.
        - If the previous step fails to find a protocol handler, then the
            constructor tries to load a built-in protocol handler.
            If this class does not exist, or if the class exists but it is not a
            subclass of `URLStreamHandler`, then a
            `MalformedURLException` is thrown.
        </ol>
        
        Protocol handlers for the following protocols are guaranteed
        to exist on the search path:
        
        - `http`
        - `https`
        - `file`
        - `jar`
        
        Protocol handlers for additional protocols may also be  available.
        Some protocol handlers, for example those used for loading platform
        classes or classes on the class path, may not be overridden. The details
        of such restrictions, and when those restrictions apply (during
        initialization of the runtime for example), are implementation specific
        and therefore not specified
        
        No validation of the inputs is performed by this constructor.

        Arguments
        - protocol: the name of the protocol to use.
        - host: the name of the host.
        - port: the port number on the host.
        - file: the file on the host

        Raises
        - MalformedURLException: if an unknown protocol or the port
                         is a negative number other than -1

        See
        - java.net.URLStreamHandlerFactory.createURLStreamHandler(
                         java.lang.String)
        """
        ...


    def __init__(self, protocol: str, host: str, file: str):
        """
        Creates a URL from the specified `protocol`
        name, `host` name, and `file` name. The
        default port for the specified protocol is used.
        
        This constructor is equivalent to the four-argument
        constructor with the only difference of using the
        default port for the specified protocol.
        
        No validation of the inputs is performed by this constructor.

        Arguments
        - protocol: the name of the protocol to use.
        - host: the name of the host.
        - file: the file on the host.

        Raises
        - MalformedURLException: if an unknown protocol is specified.

        See
        - java.net.URL.URL(java.lang.String, java.lang.String,
                         int, java.lang.String)
        """
        ...


    def __init__(self, protocol: str, host: str, port: int, file: str, handler: "URLStreamHandler"):
        """
        Creates a `URL` object from the specified
        `protocol`, `host`, `port`
        number, `file`, and `handler`. Specifying
        a `port` number of `-1` indicates that
        the URL should use the default port for the protocol. Specifying
        a `handler` of `null` indicates that the URL
        should use a default stream handler for the protocol, as outlined
        for:
            java.net.URL.URL(java.lang.String, java.lang.String, int,
                             java.lang.String)
        
        If the handler is not null and there is a security manager,
        the security manager's `checkPermission`
        method is called with a
        `NetPermission("specifyStreamHandler")` permission.
        This may result in a SecurityException.
        
        No validation of the inputs is performed by this constructor.

        Arguments
        - protocol: the name of the protocol to use.
        - host: the name of the host.
        - port: the port number on the host.
        - file: the file on the host
        - handler: the stream handler for the URL.

        Raises
        - MalformedURLException: if an unknown protocol or the port
                           is a negative number other than -1
        - SecurityException: if a security manager exists and its
               `checkPermission` method doesn't allow
               specifying a stream handler explicitly.

        See
        - java.net.NetPermission
        """
        ...


    def __init__(self, spec: str):
        """
        Creates a `URL` object from the `String`
        representation.
        
        This constructor is equivalent to a call to the two-argument
        constructor with a `null` first argument.

        Arguments
        - spec: the `String` to parse as a URL.

        Raises
        - MalformedURLException: if no protocol is specified, or an
                      unknown protocol is found, or `spec` is `null`,
                      or the parsed URL fails to comply with the specific syntax
                      of the associated protocol.

        See
        - java.net.URL.URL(java.net.URL, java.lang.String)
        """
        ...


    def __init__(self, context: "URL", spec: str):
        """
        Creates a URL by parsing the given spec within a specified context.
        
        The new URL is created from the given context URL and the spec
        argument as described in
        RFC2396 &quot;Uniform Resource Identifiers : Generic * Syntax&quot; :
        <blockquote>```
                 &lt;scheme&gt;://&lt;authority&gt;&lt;path&gt;?&lt;query&gt;#&lt;fragment&gt;
        ```</blockquote>
        The reference is parsed into the scheme, authority, path, query and
        fragment parts. If the path component is empty and the scheme,
        authority, and query components are undefined, then the new URL is a
        reference to the current document. Otherwise, the fragment and query
        parts present in the spec are used in the new URL.
        
        If the scheme component is defined in the given spec and does not match
        the scheme of the context, then the new URL is created as an absolute
        URL based on the spec alone. Otherwise the scheme component is inherited
        from the context URL.
        
        If the authority component is present in the spec then the spec is
        treated as absolute and the spec authority and path will replace the
        context authority and path. If the authority component is absent in the
        spec then the authority of the new URL will be inherited from the
        context.
        
        If the spec's path component begins with a slash character
        &quot;/&quot; then the
        path is treated as absolute and the spec path replaces the context path.
        
        Otherwise, the path is treated as a relative path and is appended to the
        context path, as described in RFC2396. Also, in this case,
        the path is canonicalized through the removal of directory
        changes made by occurrences of &quot;..&quot; and &quot;.&quot;.
        
        For a more detailed description of URL parsing, refer to RFC2396.

        Arguments
        - context: the context in which to parse the specification.
        - spec: the `String` to parse as a URL.

        Raises
        - MalformedURLException: if no protocol is specified, or an
                      unknown protocol is found, or `spec` is `null`,
                      or the parsed URL fails to comply with the specific syntax
                      of the associated protocol.

        See
        - java.net.URLStreamHandler.parseURL(java.net.URL,
                         java.lang.String, int, int)
        """
        ...


    def __init__(self, context: "URL", spec: str, handler: "URLStreamHandler"):
        """
        Creates a URL by parsing the given spec with the specified handler
        within a specified context. If the handler is null, the parsing
        occurs as with the two argument constructor.

        Arguments
        - context: the context in which to parse the specification.
        - spec: the `String` to parse as a URL.
        - handler: the stream handler for the URL.

        Raises
        - MalformedURLException: if no protocol is specified, or an
                      unknown protocol is found, or `spec` is `null`,
                      or the parsed URL fails to comply with the specific syntax
                      of the associated protocol.
        - SecurityException: if a security manager exists and its
               `checkPermission` method doesn't allow
               specifying a stream handler.

        See
        - java.net.URLStreamHandler.parseURL(java.net.URL,
                         java.lang.String, int, int)
        """
        ...


    def getQuery(self) -> str:
        """
        Gets the query part of this `URL`.

        Returns
        - the query part of this `URL`,
        or <CODE>null</CODE> if one does not exist

        Since
        - 1.3
        """
        ...


    def getPath(self) -> str:
        """
        Gets the path part of this `URL`.

        Returns
        - the path part of this `URL`, or an
        empty string if one does not exist

        Since
        - 1.3
        """
        ...


    def getUserInfo(self) -> str:
        """
        Gets the userInfo part of this `URL`.

        Returns
        - the userInfo part of this `URL`, or
        <CODE>null</CODE> if one does not exist

        Since
        - 1.3
        """
        ...


    def getAuthority(self) -> str:
        """
        Gets the authority part of this `URL`.

        Returns
        - the authority part of this `URL`

        Since
        - 1.3
        """
        ...


    def getPort(self) -> int:
        """
        Gets the port number of this `URL`.

        Returns
        - the port number, or -1 if the port is not set
        """
        ...


    def getDefaultPort(self) -> int:
        """
        Gets the default port number of the protocol associated
        with this `URL`. If the URL scheme or the URLStreamHandler
        for the URL do not define a default port number,
        then -1 is returned.

        Returns
        - the port number

        Since
        - 1.4
        """
        ...


    def getProtocol(self) -> str:
        """
        Gets the protocol name of this `URL`.

        Returns
        - the protocol of this `URL`.
        """
        ...


    def getHost(self) -> str:
        """
        Gets the host name of this `URL`, if applicable.
        The format of the host conforms to RFC 2732, i.e. for a
        literal IPv6 address, this method will return the IPv6 address
        enclosed in square brackets (`'['` and `']'`).

        Returns
        - the host name of this `URL`.
        """
        ...


    def getFile(self) -> str:
        """
        Gets the file name of this `URL`.
        The returned file portion will be
        the same as <CODE>getPath()</CODE>, plus the concatenation of
        the value of <CODE>getQuery()</CODE>, if any. If there is
        no query portion, this method and <CODE>getPath()</CODE> will
        return identical results.

        Returns
        - the file name of this `URL`,
        or an empty string if one does not exist
        """
        ...


    def getRef(self) -> str:
        """
        Gets the anchor (also known as the "reference") of this
        `URL`.

        Returns
        - the anchor (also known as the "reference") of this
                 `URL`, or <CODE>null</CODE> if one does not exist
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Compares this URL for equality with another object.
        
        If the given object is not a URL then this method immediately returns
        `False`.
        
        Two URL objects are equal if they have the same protocol, reference
        equivalent hosts, have the same port number on the host, and the same
        file and fragment of the file.
        
        Two hosts are considered equivalent if both host names can be resolved
        into the same IP addresses; else if either host name can't be
        resolved, the host names must be equal without regard to case; or both
        host names equal to null.
        
        Since hosts comparison requires name resolution, this operation is a
        blocking operation. 
        
        Note: The defined behavior for `equals` is known to
        be inconsistent with virtual hosting in HTTP.

        Arguments
        - obj: the URL to compare against.

        Returns
        - `True` if the objects are the same;
                 `False` otherwise.
        """
        ...


    def hashCode(self) -> int:
        """
        Creates an integer suitable for hash table indexing.
        
        The hash code is based upon all the URL components relevant for URL
        comparison. As such, this operation is a blocking operation.

        Returns
        - a hash code for this `URL`.
        """
        ...


    def sameFile(self, other: "URL") -> bool:
        """
        Compares two URLs, excluding the fragment component.
        
        Returns `True` if this `URL` and the
        `other` argument are equal without taking the
        fragment component into consideration.

        Arguments
        - other: the `URL` to compare against.

        Returns
        - `True` if they reference the same remote object;
                 `False` otherwise.
        """
        ...


    def toString(self) -> str:
        """
        Constructs a string representation of this `URL`. The
        string is created by calling the `toExternalForm`
        method of the stream protocol handler for this object.

        Returns
        - a string representation of this object.

        See
        - java.net.URLStreamHandler.toExternalForm(java.net.URL)
        """
        ...


    def toExternalForm(self) -> str:
        """
        Constructs a string representation of this `URL`. The
        string is created by calling the `toExternalForm`
        method of the stream protocol handler for this object.

        Returns
        - a string representation of this object.

        See
        - java.net.URLStreamHandler.toExternalForm(java.net.URL)
        """
        ...


    def toURI(self) -> "URI":
        """
        Returns a java.net.URI equivalent to this URL.
        This method functions in the same way as `new URI (this.toString())`.
        Note, any URL instance that complies with RFC 2396 can be converted
        to a URI. However, some URLs that are not strictly in compliance
        can not be converted to a URI.

        Returns
        - a URI instance equivalent to this URL.

        Raises
        - URISyntaxException: if this URL is not formatted strictly according to
                   RFC2396 and cannot be converted to a URI.

        Since
        - 1.5
        """
        ...


    def openConnection(self) -> "URLConnection":
        """
        Returns a java.net.URLConnection URLConnection instance that
        represents a connection to the remote object referred to by the
        `URL`.
        
        <P>A new instance of java.net.URLConnection URLConnection is
        created every time when invoking the
        java.net.URLStreamHandler.openConnection(URL)
        URLStreamHandler.openConnection(URL) method of the protocol handler for
        this URL.</P>
        
        <P>It should be noted that a URLConnection instance does not establish
        the actual network connection on creation. This will happen only when
        calling java.net.URLConnection.connect() URLConnection.connect().</P>
        
        <P>If for the URL's protocol (such as HTTP or JAR), there
        exists a public, specialized URLConnection subclass belonging
        to one of the following packages or one of their subpackages:
        java.lang, java.io, java.util, java.net, the connection
        returned will be of that subclass. For example, for HTTP an
        HttpURLConnection will be returned, and for JAR a
        JarURLConnection will be returned.</P>

        Returns
        - a java.net.URLConnection URLConnection linking
                    to the URL.

        Raises
        - IOException: if an I/O exception occurs.

        See
        - java.net.URL.URL(java.lang.String, java.lang.String,
                    int, java.lang.String)
        """
        ...


    def openConnection(self, proxy: "Proxy") -> "URLConnection":
        """
        Same as .openConnection(), except that the connection will be
        made through the specified proxy; Protocol handlers that do not
        support proxying will ignore the proxy parameter and make a
        normal connection.
        
        Invoking this method preempts the system's default
        java.net.ProxySelector ProxySelector settings.

        Arguments
        - proxy: the Proxy through which this connection
                    will be made. If direct connection is desired,
                    Proxy.NO_PROXY should be specified.

        Returns
        - a `URLConnection` to the URL.

        Raises
        - IOException: if an I/O exception occurs.
        - SecurityException: if a security manager is present
                    and the caller doesn't have permission to connect
                    to the proxy.
        - IllegalArgumentException: will be thrown if proxy is null,
                    or proxy has the wrong type
        - UnsupportedOperationException: if the subclass that
                    implements the protocol handler doesn't support
                    this method.

        See
        - java.net.URLStreamHandler.openConnection(java.net.URL,
                    java.net.Proxy)

        Since
        - 1.5
        """
        ...


    def openStream(self) -> "InputStream":
        """
        Opens a connection to this `URL` and returns an
        `InputStream` for reading from that connection. This
        method is a shorthand for:
        <blockquote>```
            openConnection().getInputStream()
        ```</blockquote>

        Returns
        - an input stream for reading from the URL connection.

        Raises
        - IOException: if an I/O exception occurs.

        See
        - java.net.URLConnection.getInputStream()
        """
        ...


    def getContent(self) -> "Object":
        """
        Gets the contents of this URL. This method is a shorthand for:
        <blockquote>```
            openConnection().getContent()
        ```</blockquote>

        Returns
        - the contents of this URL.

        Raises
        - IOException: if an I/O exception occurs.

        See
        - java.net.URLConnection.getContent()
        """
        ...


    def getContent(self, classes: list[type[Any]]) -> "Object":
        """
        Gets the contents of this URL. This method is a shorthand for:
        <blockquote>```
            openConnection().getContent(classes)
        ```</blockquote>

        Arguments
        - classes: an array of Java types

        Returns
        - the content object of this URL that is the first match of
                      the types specified in the classes array.
                      null if none of the requested types are supported.

        Raises
        - IOException: if an I/O exception occurs.

        See
        - java.net.URLConnection.getContent(Class[])

        Since
        - 1.3
        """
        ...


    @staticmethod
    def setURLStreamHandlerFactory(fac: "URLStreamHandlerFactory") -> None:
        """
        Sets an application's `URLStreamHandlerFactory`.
        This method can be called at most once in a given Java Virtual
        Machine.
        
         The `URLStreamHandlerFactory` instance is used to
        construct a stream protocol handler from a protocol name.
        
         If there is a security manager, this method first calls
        the security manager's `checkSetFactory` method
        to ensure the operation is allowed.
        This could result in a SecurityException.

        Arguments
        - fac: the desired factory.

        Raises
        - Error: if the application has already set a factory.
        - SecurityException: if a security manager exists and its
                    `checkSetFactory` method doesn't allow
                    the operation.

        See
        - SecurityManager.checkSetFactory
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.tags.CustomItemTagContainer

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import NamespacedKey
from org.bukkit.inventory.meta import ItemMeta
from org.bukkit.inventory.meta.tags import *
from typing import Any, Callable, Iterable, Tuple


class CustomItemTagContainer:
    """
    This interface represents a map like object, capable of storing custom tags
    in it.

    Deprecated
    - this API part has been replaced by the
    org.bukkit.persistence.PersistentDataHolder API. Please use
    org.bukkit.persistence.PersistentDataHolder instead of this.
    """

    def setCustomTag(self, key: "NamespacedKey", type: "ItemTagType"["T", "Z"], value: "Z") -> None:
        """
        Stores a custom value on the ItemMeta.
        
        This API cannot be used to manipulate minecraft tags, as the values will
        be stored using your namespace. This method will override any existing
        value the meta may have stored under the provided key.
        
        Type `<T>`: the generic java type of the tag value
        
        Type `<Z>`: the generic type of the object to store

        Arguments
        - key: the key this value will be stored under
        - type: the type this item tag uses
        - value: the value stored in the tag

        Raises
        - NullPointerException: if the key is null
        - NullPointerException: if the type is null
        - NullPointerException: if the value is null. Removing a custom tag
        should be done using .removeCustomTag(org.bukkit.NamespacedKey)
        - IllegalArgumentException: if no suitable adapter will be found for
        the ItemTagType.getPrimitiveType()
        """
        ...


    def hasCustomTag(self, key: "NamespacedKey", type: "ItemTagType"["T", "Z"]) -> bool:
        """
        Returns if the item meta has a custom tag registered matching the
        provided parameters.
        
        This method will only return if the found value has the same primitive
        data type as the provided key.
        
        Storing a value using a custom ItemTagType implementation will
        not store the complex data type. Therefore storing a UUID (by storing a
        byte[]) will match hasCustomTag("key" , ItemTagType.BYTE_ARRAY).
        Likewise a stored byte[] will always match your UUID ItemTagType
        even if it is not 16 bytes long.
        
        This method is only usable for custom object keys. Overwriting existing
        tags, like the the display name, will not work as the values are stored
        using your namespace.
        
        Type `<T>`: the generic type of the stored primitive
        
        Type `<Z>`: the generic type of the eventually created complex object

        Arguments
        - key: the key the value is stored under
        - type: the type which primitive storage type has to match the value

        Returns
        - if a value

        Raises
        - NullPointerException: if the key to look up is null
        - NullPointerException: if the type to cast the found object to is
        null
        """
        ...


    def getCustomTag(self, key: "NamespacedKey", type: "ItemTagType"["T", "Z"]) -> "Z":
        """
        Returns the custom tag's value that is stored on the item.
        
        Type `<T>`: the generic type of the stored primitive
        
        Type `<Z>`: the generic type of the eventually created complex object

        Arguments
        - key: the key to look up in the custom tag map
        - type: the type the value must have and will be casted to

        Returns
        - the value or `null` if no value was mapped under the given
        value

        Raises
        - NullPointerException: if the key to look up is null
        - NullPointerException: if the type to cast the found object to is
        null
        - IllegalArgumentException: if the value exists under the given key,
        but cannot be access using the given type
        - IllegalArgumentException: if no suitable adapter will be found for
        the ItemTagType.getPrimitiveType()
        """
        ...


    def removeCustomTag(self, key: "NamespacedKey") -> None:
        """
        Removes a custom key from the item meta.

        Arguments
        - key: the key

        Raises
        - NullPointerException: if the provided key is null
        """
        ...


    def isEmpty(self) -> bool:
        """
        Returns if the container instance is empty, therefore has no entries
        inside it.

        Returns
        - the boolean
        """
        ...


    def getAdapterContext(self) -> "ItemTagAdapterContext":
        """
        Returns the adapter context this tag container uses.

        Returns
        - the tag context
        """
        ...



"""
Python module generated from Java source file org.bukkit.conversations.PlayerNamePrompt

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.conversations import *
from org.bukkit.entity import Player
from org.bukkit.plugin import Plugin
from typing import Any, Callable, Iterable, Tuple


class PlayerNamePrompt(ValidatingPrompt):
    """
    PlayerNamePrompt is the base class for any prompt that requires the player
    to enter another player's name.
    """

    def __init__(self, plugin: "Plugin"):
        ...



"""
Python module generated from Java source file java.util.regex.Pattern

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.text import Normalizer
from java.text.Normalizer import Form
from java.util import Arrays
from java.util import Iterator
from java.util import LinkedHashSet
from java.util import Locale
from java.util import NoSuchElementException
from java.util import Spliterator
from java.util.function import Predicate
from java.util.regex import *
from java.util.stream import Stream
from java.util.stream import StreamSupport
from jdk.internal.util import ArraysSupport
from typing import Any, Callable, Iterable, Tuple


class Pattern(Serializable):

    UNIX_LINES = 0x01
    """
    Enables Unix lines mode.
    
     In this mode, only the `'\n'` line terminator is recognized
    in the behavior of `.`, `^`, and `$`.
    
     Unix lines mode can also be enabled via the embedded flag
    expression&nbsp;`(?d)`.
    """
    CASE_INSENSITIVE = 0x02
    """
    Enables case-insensitive matching.
    
     By default, case-insensitive matching assumes that only characters
    in the US-ASCII charset are being matched.  Unicode-aware
    case-insensitive matching can be enabled by specifying the .UNICODE_CASE flag in conjunction with this flag.
    
     Case-insensitive matching can also be enabled via the embedded flag
    expression&nbsp;`(?i)`.
    
     Specifying this flag may impose a slight performance penalty.  
    """
    COMMENTS = 0x04
    """
    Permits whitespace and comments in pattern.
    
     In this mode, whitespace is ignored, and embedded comments starting
    with `.` are ignored until the end of a line.
    
     Comments mode can also be enabled via the embedded flag
    expression&nbsp;`(?x)`.
    """
    MULTILINE = 0x08
    """
    Enables multiline mode.
    
     In multiline mode the expressions `^` and `$` match
    just after or just before, respectively, a line terminator or the end of
    the input sequence.  By default these expressions only match at the
    beginning and the end of the entire input sequence.
    
     Multiline mode can also be enabled via the embedded flag
    expression&nbsp;`(?m)`.  
    """
    LITERAL = 0x10
    """
    Enables literal parsing of the pattern.
    
     When this flag is specified then the input string that specifies
    the pattern is treated as a sequence of literal characters.
    Metacharacters or escape sequences in the input sequence will be
    given no special meaning.
    
    The flags CASE_INSENSITIVE and UNICODE_CASE retain their impact on
    matching when used in conjunction with this flag. The other flags
    become superfluous.
    
     There is no embedded flag character for enabling literal parsing.

    Since
    - 1.5
    """
    DOTALL = 0x20
    """
    Enables dotall mode.
    
     In dotall mode, the expression `.` matches any character,
    including a line terminator.  By default this expression does not match
    line terminators.
    
     Dotall mode can also be enabled via the embedded flag
    expression&nbsp;`(?s)`.  (The `s` is a mnemonic for
    "single-line" mode, which is what this is called in Perl.)  
    """
    UNICODE_CASE = 0x40
    """
    Enables Unicode-aware case folding.
    
     When this flag is specified then case-insensitive matching, when
    enabled by the .CASE_INSENSITIVE flag, is done in a manner
    consistent with the Unicode Standard.  By default, case-insensitive
    matching assumes that only characters in the US-ASCII charset are being
    matched.
    
     Unicode-aware case folding can also be enabled via the embedded flag
    expression&nbsp;`(?u)`.
    
     Specifying this flag may impose a performance penalty.  
    """
    CANON_EQ = 0x80
    """
    Enables canonical equivalence.
    
     When this flag is specified then two characters will be considered
    to match if, and only if, their full canonical decompositions match.
    The expression `"a&#92;u030A"`, for example, will match the
    string `"&#92;u00E5"` when this flag is specified.  By default,
    matching does not take canonical equivalence into account.
    
     There is no embedded flag character for enabling canonical
    equivalence.
    
     Specifying this flag may impose a performance penalty.  
    """
    UNICODE_CHARACTER_CLASS = 0x100
    """
    Enables the Unicode version of *Predefined character classes* and
    *POSIX character classes*.
    
     When this flag is specified then the (US-ASCII only)
    *Predefined character classes* and *POSIX character classes*
    are in conformance with
    <a href="http://www.unicode.org/reports/tr18/">*Unicode Technical
    Standard #18: Unicode Regular Expressions*</a>
    *Annex C: Compatibility Properties*.
    
    The UNICODE_CHARACTER_CLASS mode can also be enabled via the embedded
    flag expression&nbsp;`(?U)`.
    
    The flag implies UNICODE_CASE, that is, it enables Unicode-aware case
    folding.
    
    Specifying this flag may impose a performance penalty.  

    Since
    - 1.7
    """


    @staticmethod
    def compile(regex: str) -> "Pattern":
        """
        Compiles the given regular expression into a pattern.

        Arguments
        - regex: The expression to be compiled

        Returns
        - the given regular expression compiled into a pattern

        Raises
        - PatternSyntaxException: If the expression's syntax is invalid
        """
        ...


    @staticmethod
    def compile(regex: str, flags: int) -> "Pattern":
        """
        Compiles the given regular expression into a pattern with the given
        flags.

        Arguments
        - regex: The expression to be compiled
        - flags: Match flags, a bit mask that may include
                .CASE_INSENSITIVE, .MULTILINE, .DOTALL,
                .UNICODE_CASE, .CANON_EQ, .UNIX_LINES,
                .LITERAL, .UNICODE_CHARACTER_CLASS
                and .COMMENTS

        Returns
        - the given regular expression compiled into a pattern with the given flags

        Raises
        - IllegalArgumentException: If bit values other than those corresponding to the defined
                 match flags are set in `flags`
        - PatternSyntaxException: If the expression's syntax is invalid
        """
        ...


    def pattern(self) -> str:
        """
        Returns the regular expression from which this pattern was compiled.

        Returns
        - The source of this pattern
        """
        ...


    def toString(self) -> str:
        """
        Returns the string representation of this pattern. This
        is the regular expression from which this pattern was
        compiled.

        Returns
        - The string representation of this pattern

        Since
        - 1.5
        """
        ...


    def matcher(self, input: "CharSequence") -> "Matcher":
        """
        Creates a matcher that will match the given input against this pattern.

        Arguments
        - input: The character sequence to be matched

        Returns
        - A new matcher for this pattern
        """
        ...


    def flags(self) -> int:
        """
        Returns this pattern's match flags.

        Returns
        - The match flags specified when this pattern was compiled
        """
        ...


    @staticmethod
    def matches(regex: str, input: "CharSequence") -> bool:
        """
        Compiles the given regular expression and attempts to match the given
        input against it.
        
         An invocation of this convenience method of the form
        
        <blockquote>```
        Pattern.matches(regex, input);```</blockquote>
        
        behaves in exactly the same way as the expression
        
        <blockquote>```
        Pattern.compile(regex).matcher(input).matches()```</blockquote>
        
         If a pattern is to be used multiple times, compiling it once and reusing
        it will be more efficient than invoking this method each time.  

        Arguments
        - regex: The expression to be compiled
        - input: The character sequence to be matched

        Returns
        - whether or not the regular expression matches on the input

        Raises
        - PatternSyntaxException: If the expression's syntax is invalid
        """
        ...


    def split(self, input: "CharSequence", limit: int) -> list[str]:
        """
        Splits the given input sequence around matches of this pattern.
        
         The array returned by this method contains each substring of the
        input sequence that is terminated by another subsequence that matches
        this pattern or is terminated by the end of the input sequence.  The
        substrings in the array are in the order in which they occur in the
        input. If this pattern does not match any subsequence of the input then
        the resulting array has just one element, namely the input sequence in
        string form.
        
         When there is a positive-width match at the beginning of the input
        sequence then an empty leading substring is included at the beginning
        of the resulting array. A zero-width match at the beginning however
        never produces such empty leading substring.
        
         The `limit` parameter controls the number of times the
        pattern is applied and therefore affects the length of the resulting
        array.
        
           - 
           If the *limit* is positive then the pattern will be applied
           at most *limit*&nbsp;-&nbsp;1 times, the array's length will be
           no greater than *limit*, and the array's last entry will contain
           all input beyond the last matched delimiter.
        
           - 
           If the *limit* is zero then the pattern will be applied as
           many times as possible, the array can have any length, and trailing
           empty strings will be discarded.
        
           - 
           If the *limit* is negative then the pattern will be applied
           as many times as possible and the array can have any length.
        
        
         The input `"boo:and:foo"`, for example, yields the following
        results with these parameters:
        
        <table class="plain" style="margin-left:2em;">
        <caption style="display:none">Split example showing regex, limit, and result</caption>
        <thead>
        <tr>
            <th scope="col">Regex</th>
            <th scope="col">Limit</th>
            <th scope="col">Result</th>
        </tr>
        </thead>
        <tbody>
        <tr><th scope="row" rowspan="3" style="font-weight:normal">:</th>
            <th scope="row" style="font-weight:normal; text-align:right; padding-right:1em">2</th>
            <td>`{ "boo", "and:foo"`}</td></tr>
        <tr><!-- : -->
            <th scope="row" style="font-weight:normal; text-align:right; padding-right:1em">5</th>
            <td>`{ "boo", "and", "foo"`}</td></tr>
        <tr><!-- : -->
            <th scope="row" style="font-weight:normal; text-align:right; padding-right:1em">-2</th>
            <td>`{ "boo", "and", "foo"`}</td></tr>
        <tr><th scope="row" rowspan="3" style="font-weight:normal">o</th>
            <th scope="row" style="font-weight:normal; text-align:right; padding-right:1em">5</th>
            <td>`{ "b", "", ":and:f", "", ""`}</td></tr>
        <tr><!-- o -->
            <th scope="row" style="font-weight:normal; text-align:right; padding-right:1em">-2</th>
            <td>`{ "b", "", ":and:f", "", ""`}</td></tr>
        <tr><!-- o -->
            <th scope="row" style="font-weight:normal; text-align:right; padding-right:1em">0</th>
            <td>`{ "b", "", ":and:f"`}</td></tr>
        </tbody>
        </table>

        Arguments
        - input: The character sequence to be split
        - limit: The result threshold, as described above

        Returns
        - The array of strings computed by splitting the input
                 around matches of this pattern
        """
        ...


    def split(self, input: "CharSequence") -> list[str]:
        """
        Splits the given input sequence around matches of this pattern.
        
         This method works as if by invoking the two-argument .split(java.lang.CharSequence, int) split method with the given input
        sequence and a limit argument of zero.  Trailing empty strings are
        therefore not included in the resulting array. 
        
         The input `"boo:and:foo"`, for example, yields the following
        results with these expressions:
        
        <table class="plain" style="margin-left:2em">
        <caption style="display:none">Split examples showing regex and result</caption>
        <thead>
        <tr>
         <th scope="col">Regex</th>
         <th scope="col">Result</th>
        </tr>
        </thead>
        <tbody>
        <tr><th scope="row" style="text-weight:normal">:</th>
            <td>`{ "boo", "and", "foo"`}</td></tr>
        <tr><th scope="row" style="text-weight:normal">o</th>
            <td>`{ "b", "", ":and:f"`}</td></tr>
        </tbody>
        </table>

        Arguments
        - input: The character sequence to be split

        Returns
        - The array of strings computed by splitting the input
                 around matches of this pattern
        """
        ...


    @staticmethod
    def quote(s: str) -> str:
        """
        Returns a literal pattern `String` for the specified
        `String`.
        
        This method produces a `String` that can be used to
        create a `Pattern` that would match the string
        `s` as if it were a literal pattern. Metacharacters
        or escape sequences in the input sequence will be given no special
        meaning.

        Arguments
        - s: The string to be literalized

        Returns
        - A literal string replacement

        Since
        - 1.5
        """
        ...


    def asPredicate(self) -> "Predicate"[str]:
        """
        Creates a predicate that tests if this pattern is found in a given input
        string.

        Returns
        - The predicate which can be used for finding a match on a
                 subsequence of a string

        See
        - Matcher.find

        Since
        - 1.8

        Unknown Tags
        - This method creates a predicate that behaves as if it creates a matcher
        from the input sequence and then calls `find`, for example a
        predicate of the form:
        ````s -> matcher(s).find();````
        """
        ...


    def asMatchPredicate(self) -> "Predicate"[str]:
        """
        Creates a predicate that tests if this pattern matches a given input string.

        Returns
        - The predicate which can be used for matching an input string
                 against this pattern.

        See
        - Matcher.matches

        Since
        - 11

        Unknown Tags
        - This method creates a predicate that behaves as if it creates a matcher
        from the input sequence and then calls `matches`, for example a
        predicate of the form:
        ````s -> matcher(s).matches();````
        """
        ...


    def splitAsStream(self, input: "CharSequence") -> "Stream"[str]:
        """
        Creates a stream from the given input sequence around matches of this
        pattern.
        
         The stream returned by this method contains each substring of the
        input sequence that is terminated by another subsequence that matches
        this pattern or is terminated by the end of the input sequence.  The
        substrings in the stream are in the order in which they occur in the
        input. Trailing empty strings will be discarded and not encountered in
        the stream.
        
         If this pattern does not match any subsequence of the input then
        the resulting stream has just one element, namely the input sequence in
        string form.
        
         When there is a positive-width match at the beginning of the input
        sequence then an empty leading substring is included at the beginning
        of the stream. A zero-width match at the beginning however never produces
        such empty leading substring.
        
         If the input sequence is mutable, it must remain constant during the
        execution of the terminal stream operation.  Otherwise, the result of the
        terminal stream operation is undefined.

        Arguments
        - input: The character sequence to be split

        Returns
        - The stream of strings computed by splitting the input
                 around matches of this pattern

        See
        - .split(CharSequence)

        Since
        - 1.8
        """
        ...



"""
Python module generated from Java source file org.bukkit.Note

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.collect import Maps
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class Note:
    """
    A note class to store a specific note.
    """

    def __init__(self, note: int):
        """
        Creates a new note.

        Arguments
        - note: Internal note id. .getId() always return this
            value. The value has to be in the interval [0;&nbsp;24].
        """
        ...


    def __init__(self, octave: int, tone: "Tone", sharped: bool):
        """
        Creates a new note.

        Arguments
        - octave: The octave where the note is in. Has to be 0 - 2.
        - tone: The tone within the octave. If the octave is 2 the note has
            to be F#.
        - sharped: Set if the tone is sharped (e.g. for F#).
        """
        ...


    @staticmethod
    def flat(octave: int, tone: "Tone") -> "Note":
        """
        Creates a new note for a flat tone, such as A-flat.

        Arguments
        - octave: The octave where the note is in. Has to be 0 - 1.
        - tone: The tone within the octave.

        Returns
        - The new note.
        """
        ...


    @staticmethod
    def sharp(octave: int, tone: "Tone") -> "Note":
        """
        Creates a new note for a sharp tone, such as A-sharp.

        Arguments
        - octave: The octave where the note is in. Has to be 0 - 2.
        - tone: The tone within the octave. If the octave is 2 the note has
            to be F#.

        Returns
        - The new note.
        """
        ...


    @staticmethod
    def natural(octave: int, tone: "Tone") -> "Note":
        """
        Creates a new note for a natural tone, such as A-natural.

        Arguments
        - octave: The octave where the note is in. Has to be 0 - 1.
        - tone: The tone within the octave.

        Returns
        - The new note.
        """
        ...


    def sharped(self) -> "Note":
        """
        Returns
        - The note a semitone above this one.
        """
        ...


    def flattened(self) -> "Note":
        """
        Returns
        - The note a semitone below this one.
        """
        ...


    def getId(self) -> int:
        """
        Returns the internal id of this note.

        Returns
        - the internal id of this note.

        Deprecated
        - Magic value
        """
        ...


    def getOctave(self) -> int:
        """
        Returns the octave of this note.

        Returns
        - the octave of this note.
        """
        ...


    def getTone(self) -> "Tone":
        """
        Returns the tone of this note.

        Returns
        - the tone of this note.
        """
        ...


    def isSharped(self) -> bool:
        """
        Returns if this note is sharped.

        Returns
        - if this note is sharped.
        """
        ...


    def getPitch(self) -> float:
        """
        Gets the pitch of this note. This is the value used with
        World.playSound or the /playsound command.

        Returns
        - the pitch
        """
        ...


    def hashCode(self) -> int:
        ...


    def equals(self, obj: "Object") -> bool:
        ...


    def toString(self) -> str:
        ...


    class Tone(Enum):
        """
        An enum holding tones.
        """

# Static fields
        TONES_COUNT = 12
        """
        The number of tones including sharped tones.
        """


        G = (0x1, True)
        A = (0x3, True)
        B = (0x5, False)
        C = (0x6, True)
        D = (0x8, True)
        E = (0xA, False)
        F = (0xB, True)


        def getId(self) -> int:
            """
            Returns the not sharped id of this tone.

            Returns
            - the not sharped id of this tone.

            Deprecated
            - Magic value
            """
            ...


        def getId(self, sharped: bool) -> int:
            """
            Returns the id of this tone. These method allows to return the
            sharped id of the tone. If the tone couldn't be sharped it always
            return the not sharped id of this tone.

            Arguments
            - sharped: Set to True to return the sharped id.

            Returns
            - the id of this tone.

            Deprecated
            - Magic value
            """
            ...


        def isSharpable(self) -> bool:
            """
            Returns if this tone could be sharped.

            Returns
            - if this tone could be sharped.
            """
            ...


        def isSharped(self, id: int) -> bool:
            """
            Returns if this tone id is the sharped id of the tone.

            Arguments
            - id: the id of the tone.

            Returns
            - if the tone id is the sharped id of the tone.

            Raises
            - IllegalArgumentException: if neither the tone nor the
                semitone have the id.

            Deprecated
            - Magic value
            """
            ...


        @staticmethod
        def getById(id: int) -> "Tone":
            """
            Returns the tone to id. Also returning the semitones.

            Arguments
            - id: the id of the tone.

            Returns
            - the tone to id.

            Deprecated
            - Magic value
            """
            ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.BookMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from net.md_5.bungee.api.chat import BaseComponent
from org.bukkit import Material
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class BookMeta(WritableBookMeta):
    """
    Represents a Material.WRITTEN_BOOK) that can have a title, an author,
    and pages.
    """

    def hasTitle(self) -> bool:
        """
        Checks for the existence of a title in the book.

        Returns
        - True if the book has a title
        """
        ...


    def getTitle(self) -> str:
        """
        Gets the title of the book.
        
        Plugins should check that hasTitle() returns True before calling this
        method.

        Returns
        - the title of the book
        """
        ...


    def setTitle(self, title: str) -> bool:
        """
        Sets the title of the book.
        
        Limited to 32 characters. Removes title when given null.

        Arguments
        - title: the title to set

        Returns
        - True if the title was successfully set
        """
        ...


    def hasAuthor(self) -> bool:
        """
        Checks for the existence of an author in the book.

        Returns
        - True if the book has an author
        """
        ...


    def getAuthor(self) -> str:
        """
        Gets the author of the book.
        
        Plugins should check that hasAuthor() returns True before calling this
        method.

        Returns
        - the author of the book
        """
        ...


    def setAuthor(self, author: str) -> None:
        """
        Sets the author of the book. Removes author when given null.

        Arguments
        - author: the author to set
        """
        ...


    def hasGeneration(self) -> bool:
        """
        Checks for the existence of generation level in the book.

        Returns
        - True if the book has a generation level
        """
        ...


    def getGeneration(self) -> "Generation":
        """
        Gets the generation of the book.
        
        Plugins should check that hasGeneration() returns True before calling
        this method.

        Returns
        - the generation of the book
        """
        ...


    def setGeneration(self, generation: "Generation") -> None:
        """
        Sets the generation of the book. Removes generation when given null.

        Arguments
        - generation: the generation to set
        """
        ...


    def clone(self) -> "BookMeta":
        ...


    def spigot(self) -> "Spigot":
        ...


    class Spigot:

        def getPage(self, page: int) -> list["BaseComponent"]:
            """
            Gets the specified page in the book. The given page must exist.

            Arguments
            - page: the page number to get

            Returns
            - the page from the book
            """
            ...


        def setPage(self, page: int, *data: Tuple["BaseComponent", ...]) -> None:
            """
            Sets the specified page in the book. Pages of the book must be
            contiguous.
            
            The data can be up to 256 characters in length, additional characters
            are truncated.

            Arguments
            - page: the page number to set
            - data: the data to set for that page
            """
            ...


        def getPages(self) -> list[list["BaseComponent"]]:
            """
            Gets all the pages in the book.

            Returns
            - list of all the pages in the book
            """
            ...


        def setPages(self, pages: list[list["BaseComponent"]]) -> None:
            """
            Clears the existing book pages, and sets the book to use the provided
            pages. Maximum 50 pages with 256 characters per page.

            Arguments
            - pages: A list of pages to set the book to use
            """
            ...


        def setPages(self, *pages: Tuple[list["BaseComponent"], ...]) -> None:
            """
            Clears the existing book pages, and sets the book to use the provided
            pages. Maximum 50 pages with 256 characters per page.

            Arguments
            - pages: A list of component arrays, each being a page
            """
            ...


        def addPage(self, *pages: Tuple[list["BaseComponent"], ...]) -> None:
            """
            Adds new pages to the end of the book. Up to a maximum of 50 pages
            with 256 characters per page.

            Arguments
            - pages: A list of component arrays, each being a page
            """
            ...



"""
Python module generated from Java source file org.bukkit.help.IndexHelpTopic

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import ChatColor
from org.bukkit.command import CommandSender
from org.bukkit.command import ConsoleCommandSender
from org.bukkit.entity import Player
from org.bukkit.help import *
from org.bukkit.util import ChatPaginator
from typing import Any, Callable, Iterable, Tuple


class IndexHelpTopic(HelpTopic):
    """
    This help topic generates a list of other help topics. This class is useful
    for adding your own index help topics. To enforce a particular order, use a
    sorted collection.
    
    If a preamble is provided to the constructor, that text will be displayed
    before the first item in the index.
    """

    def __init__(self, name: str, shortText: str, permission: str, topics: Iterable["HelpTopic"]):
        ...


    def __init__(self, name: str, shortText: str, permission: str, topics: Iterable["HelpTopic"], preamble: str):
        ...


    def canSee(self, sender: "CommandSender") -> bool:
        ...


    def amendCanSee(self, amendedPermission: str) -> None:
        ...


    def getFullText(self, sender: "CommandSender") -> str:
        ...



"""
Python module generated from Java source file org.bukkit.entity.AbstractVillager

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from org.bukkit.inventory import Inventory
from org.bukkit.inventory import InventoryHolder
from org.bukkit.inventory import Merchant
from typing import Any, Callable, Iterable, Tuple


class AbstractVillager(Breedable, NPC, InventoryHolder, Merchant):
    """
    Represents a villager NPC
    """

    def getInventory(self) -> "Inventory":
        """
        Gets this villager's inventory.
        
        Note that this inventory is not the Merchant inventory, rather, it is the
        items that a villager might have collected (from harvesting crops, etc.)
        
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.Explosive

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import *
from typing import Any, Callable, Iterable, Tuple


class Explosive(Entity):
    """
    A representation of an explosive entity
    """

    def setYield(self, yield: float) -> None:
        """
        Set the radius affected by this explosive's explosion.
        
        This is the base yield, which may be affected by other entity attributes.

        Arguments
        - yield: The explosive yield
        """
        ...


    def getYield(self) -> float:
        """
        Return the radius or yield of this explosive's explosion.
        
        This is the base yield, which may be affected by other entity attributes.

        Returns
        - the radius of blocks affected
        """
        ...


    def setIsIncendiary(self, isIncendiary: bool) -> None:
        """
        Set whether or not this explosive's explosion causes fire

        Arguments
        - isIncendiary: Whether it should cause fire
        """
        ...


    def isIncendiary(self) -> bool:
        """
        Return whether or not this explosive creates a fire when exploding

        Returns
        - True if the explosive creates fire, False otherwise
        """
        ...



"""
Python module generated from Java source file com.google.common.base.VerifyException

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.base import *
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class VerifyException(RuntimeException):
    """
    Exception thrown upon the failure of a <a
    href="https://github.com/google/guava/wiki/ConditionalFailuresExplained">verification check</a>,
    including those performed by the convenience methods of the Verify class.

    Since
    - 17.0
    """

    def __init__(self):
        """
        Constructs a `VerifyException` with no message.
        """
        ...


    def __init__(self, message: str):
        """
        Constructs a `VerifyException` with the message `message`.
        """
        ...


    def __init__(self, cause: "Throwable"):
        """
        Constructs a `VerifyException` with the cause `cause` and a message that is `null` if `cause` is null, and `cause.toString()` otherwise.

        Since
        - 19.0
        """
        ...


    def __init__(self, message: str, cause: "Throwable"):
        """
        Constructs a `VerifyException` with the message `message` and the cause `cause`.

        Since
        - 19.0
        """
        ...



"""
Python module generated from Java source file org.yaml.snakeyaml.nodes.MappingNode

Java source file obtained from artifact snakeyaml version 2.2

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.yaml.snakeyaml import DumperOptions
from org.yaml.snakeyaml.error import Mark
from org.yaml.snakeyaml.nodes import *
from typing import Any, Callable, Iterable, Tuple


class MappingNode(CollectionNode):
    """
    Represents a map.
    
    A map is a collection of unsorted key-value pairs.
    """

    def __init__(self, tag: "Tag", resolved: bool, value: list["NodeTuple"], startMark: "Mark", endMark: "Mark", flowStyle: "DumperOptions.FlowStyle"):
        ...


    def __init__(self, tag: "Tag", value: list["NodeTuple"], flowStyle: "DumperOptions.FlowStyle"):
        ...


    def getNodeId(self) -> "NodeId":
        ...


    def getValue(self) -> list["NodeTuple"]:
        """
        Returns the entries of this map.

        Returns
        - List of entries.
        """
        ...


    def setValue(self, mergedValue: list["NodeTuple"]) -> None:
        ...


    def setOnlyKeyType(self, keyType: type["Object"]) -> None:
        ...


    def setTypes(self, keyType: type["Object"], valueType: type["Object"]) -> None:
        ...


    def toString(self) -> str:
        ...


    def setMerged(self, merged: bool) -> None:
        """
        Arguments
        - merged: - True if map contains merge node
        """
        ...


    def isMerged(self) -> bool:
        """
        Returns
        - True if map contains merge node
        """
        ...
